# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from gist import *
from shapetest import *
from Numeric import *
from scipy_base.fastumath import *
from graftypes import *
from movie import *
from pl3d import *
from plwf import *
from slice3 import *
from colorbar import *
from mesh3d import *
from arrayfns import *
 
def minmax ( x ) :
   """minmax (x) where x is a two-dimensional array computes the minimum and
   maximum values in the array and returns them as a list [min, max]. I use
   this routine because there are apparently some circumstances in which Gist
   fails to calculate default axis limits correctly.
   """
   return [floor (min (ravel (x))), ceil (max (ravel (x)))]

def true_minmax ( x ) :
   return [min (ravel (x)), max (ravel (x))]

class Plotter :
   """Plotter ( <name> , <keyword arguments> ) creates a Gist Plotter
   object. The <name>, if non-blank, specifies where the display is to
   occur; if blank, it will be your default display; and if null (""),
   there will be no window at all. The keywords
   and their defaults are as follows:
     n (0) -- the number of the graphics window (0 to 7 are allowed).
              each plotter object corresponds to a separate window.
     dpi (100) -- the size of the window wanted. 100 and 75 are allowed;
                  100 is the larger size.
     wait (1) -- used to make sure everything is plotted before
                 changing frames.
     private (0) -- use a common colormap.
     hcp -- if not present, use default hardcopy file used by all
            windows. If present, names a file unique to this window.
     dump (0) -- if 1, dumps the color palette at the beginning of
                 each page of hardcopy output, otherwise converts
                 to grey scale.
     legends (0) -- controls whether (1) or not (0) curve legends
                    are dumped to the hardcopy.
     style ("work.gs") -- name of a Gist style sheet.
   """

   def type (self) :
      return GistType
 
   def _set_defaults ( self ) :
      self._n = -1
      self._dpi = 100
      self._wait = 1
      self._private = 0
      self._dump = 0
      self._legends = 0
      self._style = "work.gs"
      self._xscale = 0 # 0 for linear, 1 for log
      self._yscale = 0 # 0 for linear, 1 for log
      self._xmin = "e"
      self._xmax = "e"
      self._ymin = "e"
      self._ymax = "e"
      self._xmindefault = 1
      self._ymindefault = 1
      self._xmaxdefault = 1
      self._ymaxdefault = 1
      self._text = []
      self._text_color = []
      self._text_size = []
      self._text_pos = []
      self._tosys = []
      self._xyequal = 0
      self._cmin = 0.0
      self._cmax = 0.0
      self._z_c_switch = None
      # (ZCM 3/26/97) Added this so that plmesh need be called
      # only once for a given quadmesh.
      self._quadmesh = None
      self._current_palette = None
      self.making_movie = 0
      self.equal_axes = 0

   from os import environ

   _DisplayError = "DisplayError"

   def __init__ ( self , filename = ' ' , ** kw ) :
      self._set_defaults ( )
      for k in kw.keys ( ) :
         setattr ( self , "_" + k , kw [k] )
      # if there is a window with this number, kill it and its hard copy.
      if (self._n == -1) : # no number specified so use 0
         winkill (0)
         self._n = 0
      else :
         winkill ( self._n )
      if filename == " " and hasattr (self, "_display") :
         filename = self._display
      if filename == " " :
         try :
            filename = self.environ ["DISPLAY"]
         except :
            raise self._DisplayError, \
               "Gist does not know where to display your plots.\n" + \
               "You need to exit python and set your DISPLAY variable,\n" + \
               "or else instantiate your Graph object using the 'display'\n" + \
               "keyword argument."
      if (filename == "none" or filename == "" or filename is None) and \
         hasattr ( self , "_hcp" ) :
         window ( self._n , display = "" , hcp = self._hcp ,  dpi = self._dpi ,
                  wait = self._wait , legends = self._legends ,
                  style = self._style )
         hcpon ()
      elif hasattr ( self , "_hcp" ) and self._hcp != "" and filename != " " :
         hcp_file (self._hcp)
         window ( self._n , display = filename , dpi = self._dpi ,
                  wait = self._wait , private = self._private ,
                  hcp = self._hcp , dump = self._dump ,
                  legends = self._legends , style = self._style )
         hcpon ()
      elif hasattr( self , "_hcp" ) and self._hcp != "" and filename == " " :
         hcp_file (self._hcp)
         window ( self._n , dpi = self._dpi , wait = self._wait ,
                  private = self._private , hcp = self._hcp ,
                  dump = self._dump , legends = self._legends ,
                  style = self._style )
         hcpon ()
      elif filename != " " :
         window ( self._n ,  display = filename , dpi = self._dpi ,
                  wait = self._wait , private = self._private ,
                  legends = self._legends , style = self._style )
      else :
         window ( self._n , dpi = self._dpi , wait = self._wait ,
                  private = self._private ,
                  legends = self._legends , style = self._style )
      self._filename = filename
      if filename != "none" and filename != "" :
         print "Opening graphics window " + `self._n` + "."
      self._open = 1
      self._GistError = "GistError"
 
   def close ( self ) :
      "close () closes the connection to Gist."
      if self._open :
         if (self._filename == "none" or self._filename == "") \
            and hasattr ( self , "_hcp" ) :
            if window () != self._n :
               window ( self._n )
            hcp ( ) # Make sure last frame goes out
            fma ( )
            print "Closing hard copy file " + hcp_finish (self._n) + "."
            winkill ( self._n )
         else :
            winkill ( self._n )
            print "Closing graphics window " + `self._n` + "."
         self._open = 0
 
   def __del__ ( self ) :
      self.close ( )
 
   def query ( self ) :
       if not self._open :
          return -1
       else :
          return 1

   narcisse_to_gist_cc_dict = { "absolute" : "gray.gp" , "binary" : "gray.gp" ,
       "bluegreen" : "stern.gp" , "default" : "rainbow.gp" ,
       "negative" : "yarg.gp" , "positive" : "gray.gp" ,
       "rainbow" : "rainbow.gp" , "rainbowhls" : "rainbow.gp" ,
       "random" : "earth.gp" , "redblue" : "heat.gp" , "redgreen" : "heat.gp" ,
       "shifted" : "earth.gp" , 0 : "gray.gp" , 1 : "gray.gp" , 2 : "stern.gp" ,
        3 : "gray.gp" , 4 : "yarg.gp" , 5 : "gray.gp" , 6 : "rainbow.gp" ,
        7 : "rainbow.gp" , 8 : "earth.gp" , 9 : "heat.gp" , 10 : "heat.gp" ,
        11 : "earth.gp" }

   _legal_card = { 'earth.gp' : 1, 'stern.gp' : 1, 'rainbow.gp' : 1,
                   'heat.gp' : 1, 'gray.gp' : 1, 'yarg.gp' : 1 }

   def set_style ( self, st ) :
       """set_style ( st ) sets the style sheet to a new value"""
       self._style = st
       window (style = st)

   def set_distance ( self, * val ) :
       """set_distance (d) sets the viewing distance depending on d,
       a number between 0 and 20. 0 signifies infinite distance;
       20 signals very close. These need to be translated to true
       distance values for Gist.
       """
       if len (val) == 0 or val [0] <= 0 or val [0] >= 20 :
          setz3 (None)
       else :
          if val [0] < 0.1 :
             setz3 (1.e6)
          else :
             setz3 ( - 50251.206 * val [0] + 1005025.1206)

   def set_bytscl ( self, cmin, cmax ) :
       """set_bytscl ( cmin, cmax ) sets _cmin and _cmax so that
       bytscl will be called for the next plf command.
       """
       self._cmin = cmin
       self._cmax = cmax

   def set_color_card ( self , cd , now = 0) :
       """set_color_card ( palette ) indicates a predefined color card
       for a plot. Standard ones are 'earth.gp', 'stern.gp',
       'rainbow.gp', 'heat.gp', gray.gp', and 'yarg.gp'. If a Narcisse
       color card is specified, we do a translation. If an illegal one
       is specified, use the default.
       """
       hh = cd
       if self.narcisse_to_gist_cc_dict.has_key (cd) :
          hh = self.narcisse_to_gist_cc_dict [cd]
       elif not self._legal_card.has_key (cd) :
          print "Gist warning:" , cd , "is not a legal Gist color card."
          hh = "rainbow.gp"
       self._palette = hh
       if now == 1 :
          palette ( self._palette )
          del self._palette

   def send_color_card ( self ) :
       if hasattr (self, "_palette") :
          if self._current_palette != self._palette :
             palette ( self._palette )
             self._current_palette = self._palette
             del self._palette

   def set_titles ( self , * vals ) :
       """set_titles ('bottom', 'top', 'left', 'right')
       All arguments are optional. Missing ones default to ' '.
       Gist uses plt to plot the titles; values must be saved till
       the graph is plotted.
       """
       if len ( vals ) == 0 or len ( vals [0] ) == 0 :
          self._titles = [" ", " ", " ", " "]
       elif is_scalar ( vals [0] ) :
          if len (vals) == 1 :
             self._titles = [vals [0]] + [" ", " ", " "]
          elif len (vals) == 2 :
             self._titles = [vals [0], vals [1]] + [" ", " "]
          elif len (vals) == 3 :
             self._titles = [vals [0], vals [1], vals [2]] + [" "]
          else :
             self._titles = [vals [0], vals [1], vals [2], vals [3]]
       elif len ( vals[0] ) == 1 :
          self._titles = vals[0] + [" ", " ", " "]
       elif len ( vals[0] ) == 2 :
          self._titles = vals[0] + [" ", " "]
       elif len ( vals[0] ) == 3 :
          self._titles = vals[0] + [" "]
       else :
          self._titles = vals[0]
 
   def set_title_colors ( self , *vals ) :
       """set_title_colors (bottom_color, top_color, left_color, right_color)
         All arguments are optional, integers from 0 to 63 representing
         a color in some color map. Missing arguments default
         to foreground."""
       if len ( vals ) == 0 :
          self._title_colors = ["fg", "fg", "fg", "fg"]
       elif is_scalar ( vals[0] ) :
          if len (vals) == 1 :
             self._title_colors = [vals [0]] + ["fg", "fg", "fg"]
          elif len (vals) == 2 :
             self._title_colors = [vals [0], vals [1]] + ["fg", "fg"]
          elif len (vals) == 3 :
             self._title_colors = [vals [0], vals [1], vals [2]] + ["fg"]
          else :
             self._title_colors = [vals [0], vals [1], vals [2],vals [3]]
       elif len ( vals[0] ) == 1 :
          self._title_colors = vals[0] + ["fg", "fg", "fg"]
       elif len ( vals[0] ) == 2 :
          self._title_colors = vals[0] + ["fg", "fg"]
       elif len ( vals[0] ) == 3 :
          self._title_colors = vals[0] + ["fg"]
       else :
          self._title_colors = vals[0]

 
   def _lengthen_if_necessary ( self , n ) :
       while len (self._text) <= n :
          self._text.append ( " " )
          self._text_color.append ( "fg" )
          self._text_size.append ( 14.0 )
          self._text_pos.append ( [0., 0.] )
          self._tosys.append ( 0 )
 
   # The following five functions set various aspects of the text
   # array. In Gist, you must call plot_text in order to have the
   # text appear on the graph.
   def set_text ( self, t , n ) :
       "set_text (str, ix) sets the ix'th text to str."
       self._lengthen_if_necessary ( n )
       self._text [n] = t
       return

   def set_tosys ( self, v, n ) :
       "set_tosys ( v, n ) sets the n'th tosys to v."
       self._lengthen_if_necessary ( n )
       self._tosys [n] = v
       return

   def add_text (self, str, x, y, size, color="fg", tosys = 1) :
       """add_text (str, x, y, size [, color]) adds a text to a graph."""
       self._text.append (str)
       self._text_pos.append ([x, y])
       self._text_size.append (size)
       self._text_color.append (self._figure_color (color))
       self._tosys.append (tosys)

   def clear_text ( self ) :
       "clear_text ( ) makes sure that there is no text to be put out."
       self._text = []
       self._text_pos = []
       self._text_size = []
       self._text_color = []
       self._tosys = []
 
   def set_text_color ( self, n1, n2 ) :
       """set_text_color (col, ix) sets the ix'th text color to col"""
       self._lengthen_if_necessary ( n2 )
       self._text_color [n2] = self._figure_color (n1)
       return
 
   def set_text_size ( self , n , m ) :
       """set_text_size (sz, ix) sets the ix'th text size to sz.
       Text sizes mean different things in Gist and Narcisse, so
       the user has to be aware of this."""
       self._lengthen_if_necessary ( m )
       self._text_size [m] = n
       return
 
   def set_text_pos ( self , n, m, mm , tosys = 0) :
       """set_text_pos (x, y, ix) positions the ix'th text at (x, y),
       which are real numbers between 0 and 1 giving relative
       position in the graphics window."""
       self._lengthen_if_necessary ( mm )
       self._text_pos [mm] = [n, m]
       return

   def new_frame ( self ) :
       fma ( )
 
   def plot_text ( self ) :
       """plot_text ( ) sends the accumulated texts out to the
       graph."""
       if len ( self._text ) == 0 :
           return
       for n in range (len ( self._text )) :
          if self._text [n] != '' and self._text [n] != ' ' :
             plt ( self._text [n] , self._text_pos [n][0] ,
                   self._text_pos [n][1] , color = self._text_color [n] ,
                   height = self._text_size [n] , tosys = self._tosys [n])
       return
 
   def freeze_graph ( self ) :
       pass
 
   def set_no_concat ( self ) :
       pass
 
   def set_axis_labels ( self , x ) :
       pass

   def set_3d_grid_type (self, t) :
       pass

   def set_grid_type ( self , * val ) :
       """set_grid_type ( string ) determines how intrusive the axes
       and grids are. The legal arguments are:
       'none'--no axes and grids are drawn.
       'axes'--axes with tick marks.
       'wide'--widely spaced grid in x and y (2d or 3d).
       'full'--narrowly spaced grid in x and y (2d or 3d).
           'full' is not currently implemented.
       If no argument is specified, the default is 'axes'."""
 
       if len ( val ) > 1 :
          raise  self._GistError, "Too many arguments to set_grid_type."
       if len ( val ) == 0 or val [0] == "axes" :
          self._gridtype = 2
       elif val [0] == "none" :
          self._gridtype = 0
       elif val [0] == "wide" :
          self._gridtype = 1
 
   def set_axis_lin ( self , ax ) :
       """set_axis_lin (ax) where ax can be 'x', 'y', or 'all'.
       The specified axis will have a linear scale."""
       if ( ax == "x" ) :
          self._xscale = 0
       elif ( ax == "y" ) :
          self._yscale = 0
       elif ( ax == "all" ) :
          self._xscale = 0
          self._yscale = 0
       else :
          return
       #logxy ( self._xscale, self._yscale )
 
   def set_axis_log ( self , ax ) :
       """set_axis_log (ax) where ax can be 'x', 'y', or 'all'.
       The specified axis will have a logarithmic scale."""
       if ( ax == "x" ) :
          self._xscale = 1
       elif ( ax == "y" ) :
          self._yscale = 1
       elif ( ax == "all" ) :
          self._xscale = 1
          self._yscale = 1
       else :
          return
       #logxy ( self._xscale, self._yscale )
 
   def set_linlin ( self ) :
       "set_linlin () sets both x and y axes to linear scale."
       self._xscale = 0
       self._yscale = 0
       #logxy ( 0 , 0 )
 
   def set_linlog ( self ) :
       'set_linlog () sets x axis to linear, y axis to logarithmic.'
       self._xscale = 0
       self._yscale = 1
       #logxy ( 0 , 1 )
 
   def set_loglin ( self ) :
       'set_loglin () sets x axis to logarithmic, y axis to linear.'
       self._xscale = 1
       self._yscale = 0
       #logxy ( 1 , 0 )
 
   def set_loglog ( self ) :
       'set_loglog () sets both x and y axes to logarithmic scale.'
       self._xscale = 1
       self._yscale = 1
       #logxy ( 1 , 1 )
 
   def set_axis_max ( self , ax , * val1 ) :
       """set_axis_max (ax, val) where ax is 'x' or 'y'.
          The maximum of the specified axis will be set to val.
          val should be a PyFloat object, or, if omitted, the
          default will be chosen.
       """
       # (6/13/96) In all the axis limits routines, I make note whether
       # axis defaults have been asked for, and set flags accordingly.
       # For flags which are set, I will calculate defaults myself.
       # This is because I have found various instances in which
       # Gist fails to do so correctly. The calculation, unfortunately,
       # has to wait until curves (etc.) are sent to be plotted.
       if ax == "x" :
          if len ( val1 ) == 0 :
             self._xmax = "e"
          else :
             self._xmax = val1 [0]
          if self._xmin == 0 and ( self._xmax == 0 or self._xmax == "e" ) :
             self._xmin = "e"
             self._xmax = "e"
          if self._xmin == "e" : self._xmindefault = 1
          else :                 self._xmindefault = 0
          if self._xmax == "e" : self._xmaxdefault = 1
          else :                 self._xmaxdefault = 0
       elif ax == "y" :
          if len ( val1 ) == 0 :
             self._ymax = "e"
          else :
             self._ymax = val1 [0]
          if self._ymin == 0 and ( self._ymax == 0 or self._ymax == "e" ) :
             self._ymin = "e"
             self._ymax = "e"
          if self._ymin == "e" : self._ymindefault = 1
          else :                 self._ymindefault = 0
          if self._ymax == "e" : self._ymaxdefault = 1
          else :                 self._ymaxdefault = 0
 
   def set_axis_min ( self , ax , * val1 ) :
       """set_axis_min (ax, val) where ax is 'x' or 'y'.
          The minimum of the specified axis will be set to val.
          val should be a PyFloat object, or, if omitted, the
          default will be chosen.
       """
       if ax == "x" :
          if len ( val1 ) == 0 :
             self._xmin = "e"
          else :
             self._xmin = val1 [0]
          if self._xmin == 0 and self._xmax == 0 :
             self._xmin = "e"
             self._xmax = "e"
          if self._xmin == "e" : self._xmindefault = 1
          else :                 self._xmindefault = 0
          if self._xmax == "e" : self._xmaxdefault = 1
          else :                 self._xmaxdefault = 0
       elif ax == "y" :
          if len ( val1 ) == 0 :
             self._ymin = "e"
          else :
             self._ymin = val1 [0]
          if self._ymin == 0 and self._ymax == 0 :
             self._ymin = "e"
             self._ymax = "e"
          if self._ymin == "e" : self._ymindefault = 1
          else :                 self._ymindefault = 0
          if self._ymax == "e" : self._ymaxdefault = 1
          else :                 self._ymaxdefault = 0
 
   def set_default_axes_limits ( self , * h ) :
       '''set_default_axes_limits () sets Gist to compute the maximum
       and minimum of the axes depending on the data.'''
 
       self._xmin = "e"
       self._xmax = "e"
       self._ymin = "e"
       self._ymax = "e"
       self._xmindefault = 1
       self._ymindefault = 1
       self._xmaxdefault = 1
       self._ymaxdefault = 1
 
   def set_x_axis_limits ( self , val1 , * val2i ) :
       '''set_x_axis_limits (min, max) sets the limits on the x axis to
       the specified (pyFloat) sizes. If max is missing, take the default.'''
       self._xmin = val1
       if len (val2i) == 0:
          self._xmax = "e"
       else :
          self._xmax = val2i [0]
       if self._xmin == 0 and ( self._xmax == 0 or self._xmax == "e" ) :
          self._xmin = "e"
          self._xmax = "e"
       if self._xmin == "e" : self._xmindefault = 1
       else :                 self._xmindefault = 0
       if self._xmax == "e" : self._xmaxdefault = 1
       else :                 self._xmaxdefault = 0
 
   def set_y_axis_limits ( self , val1 , * val2i ) :
       '''set_y_axis_limits (min, max) sets the limits on the x axis to
       the specified (pyFloat) sizes. If max is missing, take the default.'''
       self._ymin = val1
       if len (val2i) == 0:
          self._ymax = "e"
       else :
          self._ymax = val2i [0]
       if self._ymin == 0 and ( self._ymax == 0 or self._ymax == "e" ) :
          self._ymin = "e"
          self._ymax = "e"
       if self._ymin == "e" : self._ymindefault = 1
       else :                 self._ymindefault = 0
       if self._ymax == "e" : self._ymaxdefault = 1
       else :                 self._ymaxdefault = 0

   def set_z_axis_limits ( self, val1, val2 ) :
       '''set_z_axis_limits (min, max) sets the limits on the z axis to
       the specified (pyFloat) sizes. If val1 and val2 are both 0.0,
       we assume default calculations; otherwise, graph only the part
       of the surface whose z coordinate is between val1 and val2.'''
       if val1 == val2 == 0.0 :
          self._zmin = self._zmax = None
       else :
          self._zmin = val1
          self._zmax = val2

   def set_c_axis_limits ( self, val1, val2 ) :
       '''set_c_axis_limits (min, max) sets the limits on the c values to
       the specified (pyFloat) sizes. If val1 and val2 are both 0.0,
       we assume default calculations; otherwise, graph only the part
       of the surface whose c value is between val1 and val2.'''
       if val1 == val2 == 0.0 :
          self._cmin = self._cmax = None
       else :
          self._cmin = val1
          self._cmax = val2

   def set_xyequal ( self ) :
       self._xyequal = 1

   def reset_xyequal ( self ) :
       self._xyequal = 0
 
   # Translation table from Narcisse colors to gist
   narcisse_to_gist_col = {
                   "foreground" : "fg",
                   "background" : "bg" , "orange" : "yellow" ,
                   "purple" : "magenta" }
   legal_gist_colors = { "bg" : 1 , "fg" : 1 , "black" : 1 , "white" : 1 ,
                         "red" : 1 , "green" : 1 , "blue" : 1 , "cyan" : 1 ,
                         "magenta" : 1 , "yellow" : 1 }

   def _figure_color ( self, col ) :
       """_figure_color ( col ) does the best job it can to return
       a correct color. If the value is legal for Gist (even though
       it may mean something else in another system) then it is
       returned unchanged. If it is a Narcisse value, it is converted
       to Gist if possible. In all other cases, return 'fg.'
       """
       if type (col) == IntType and (-10 <= col <= 63 ) \
          or self.legal_gist_colors.has_key (col) : # probably OK
          return col
       if self.narcisse_to_gist_col.has_key (col) :
          return self.narcisse_to_gist_col [col]
       print "Gist warning:" , col , "is an unknown color."
       return "fg"

   legal_gist_types = { "solid" : 1 , "dash" : 1 , "dot" : 1 ,
                        "dashdot" : 1 , "dashdotdot" : 1 , "none" : 1 }
   narcisse_to_gist_tp = { "line" : "solid" , "step" : "solid" ,
                           "unspecified" : "solid" , 6 : "dot" ,
                           "dotted" : "dot"}

   def _figure_type ( self , tp ) :
       """_figure_type ( tp ) does the best job it can to return
       a correct type. If the value is legal for Gist (even though
       it may mean something else in another system) then it is
       returned unchanged. If it is a Narcisse value, it is converted
       to Gist if possible. In all other cases, return 'solid.'
       """
       if type (tp) == IntType and (0 <= tp <= 5) \
          or self.legal_gist_types.has_key (tp) :
          return tp
       if self.narcisse_to_gist_tp.has_key ( tp ) :
          return self.narcisse_to_gist_tp [tp]
       print "Gist warning:" , tp , "is an unknown curve type."
       return "solid"

   _title_coords = [[.3935, .3655], [.3935, .8963], 
                    [.1278, .6483], [.6473, .6483]]
   _just = ["CB", "CT", "LH", "RH"]

   def _plot_titles ( self ) :
       """_plot_titles ( ) plots the four titles (if any)."""
       for i in range (4) :
          if self._titles [i] != '' and self._titles [i] != ' ' :
             plt (self._titles [i], self._title_coords [i][0],
                  self._title_coords [i][1],
                  color = self._figure_color (self._title_colors [i]),
                  justify = self._just [i]) ## , path = self._paths [i])

   def plot_object ( self , crv ) :

       """plot_object (crv) is a general purpose plotting routine. It should
       be called with one argument, a curve, quadmesh, polymap, cellarray,
       or lines object. In the case of multiple objects on one graph, the
       first call only should be to this routine, subsequent calls to
       add_object. plot_object does some one-time things, such as
       plotting the titles.
       """
       # Due to Gist problems calculating default axis limits correctly,
       # I'm doing it here.
       if crv.type () == QuadMeshType :
          if self._quadmesh != crv : # plmesh it
             self._quadmesh = crv
             # (ZCM 3/26/97) Do a plmesh first, otherwise boundary
             # gets all screwed up.
             plmesh (array (crv.y).astype (Float),
                     array (crv.x).astype (Float),
                     array (crv.ireg).astype (Int),
                     triangle = array (crv.tri).astype (Float))
          if self._xmindefault or self._xmaxdefault :
             new_limits = minmax ( crv.x )
             if self._xmindefault : self._xmin = new_limits [0]
             if self._xmaxdefault : self._xmax = new_limits [1]
          if self._ymindefault or self._ymaxdefault :
             new_limits = minmax ( crv.y )
             if self._ymindefault : self._ymin = new_limits [0]
             if self._ymaxdefault : self._ymax = new_limits [1]
       self._first_curve = 1
       n = current_window ( )
       if (n == -1 or n != self._n) :
          window (self._n)
       fma ( )
       self.add_object ( crv )

   def add_object ( self , crv ) :
       """add_object (crv) simply adds a curve to an existing graph.
       In Gist graphics, this is the same as plot_object except there
       is no frame advance."""
       global _color_bar_
       if crv.type () == QuadMeshType :
          if self._first_curve == 0 :
             # May need to compute new x and y axis limits
             if self._xmindefault or self._xmaxdefault :
                new_limits = minmax ( crv.x )
                if self._xmindefault and new_limits [0] < self._xmin :
                   self._xmin = new_limits [0]
                if self._xmaxdefault and new_limits [1] > self._xmax :
                   self._xmax = new_limits [1]
             if self._ymindefault or self._ymaxdefault :
                new_limits = minmax ( crv.y )
                if self._ymindefault and new_limits [0] < self._ymin :
                   self._ymin = new_limits [0]
                if self._ymaxdefault and new_limits [1] > self._ymax :
                   self._ymax = new_limits [1]
          if self._xyequal :
             # need to revise limits in light of this requirement
             xdiff = self._xmax - self._xmin
             ydiff = self._ymax - self._ymin
             if xdiff > ydiff :
                self._ymax = self._ymax + xdiff - ydiff
             elif ydiff > xdiff :
                self._xmax = self._xmax + ydiff - xdiff
       if self._xmin == 'd':
          self._xmin = 'e'
       if self._xmax == 'd':
          self._xmax = 'e'
       if self._ymin == 'd':
          self._ymin = 'e'
       if self._ymax == 'd':
          self._ymax = 'e'
       limits ( self._xmin, self._xmax )
       ylimits ( self._ymin, self._ymax)
       if hasattr (crv, "color") :
          crv.color = self._figure_color (crv.color)
       if hasattr (crv, "line_type") :
          crv.line_type = self._figure_type (crv.line_type)
       if crv.type () == CurveType :
          if self._first_curve:
             self._first_curve = 0
             self.send_color_card ( )
             gridxy (self._gridtype)
             logxy ( self._xscale, self._yscale )
             self._plot_titles ( )
          if (crv.marks == 0 or crv.marker is None) and crv.label != " " :
             marks = 1
             marker = crv.label [0]
          else :
             marks = crv.marks
             if crv.line_type == "dot" and crv.marker == "." \
                and crv.label != " " :
                marker = crv.label [0]
             else :
                marker = crv.marker
          if marks != 0 and (marker is None or marker == " ") :
              # Give user default markers
              plg ( array (crv.y).astype (Float),
                array (crv.x).astype (Float),
                hide = crv.hide, width = crv.width, type = crv.line_type,
                color = crv.color, marks = marks)
          else :
             plg ( array (crv.y).astype (Float),
                array (crv.x).astype (Float),
                hide = crv.hide, width = crv.width, type = crv.line_type,
                color = crv.color, marks = marks,
                marker = marker )
       elif crv.type () == LinesType :
          if self._first_curve:
             self._first_curve = 0
             self.send_color_card ( )
             gridxy (self._gridtype)
             logxy ( self._xscale, self._yscale )
             self._plot_titles ( )
          pldj ( array (crv.x0).astype (Float), array (crv.y0).astype (Float),
                 array (crv.x1).astype (Float), array (crv.y1).astype (Float),
                 hide = crv.hide, width = crv.width,
                 type = crv.line_type, color = crv.color )
       elif crv.type () == PolyMapType :
          if self._first_curve:
             self._first_curve = 0
             self.send_color_card ( )
             gridxy (self._gridtype)
             logxy ( self._xscale, self._yscale )
             self._plot_titles ( )
          if _color_bar_ :
             self.set_style ("z_work.gs")
             color_bar (min (ravel (crv.z)), max (ravel (crv.z)), 0)
          if type (crv.z[0]) == type (1.0) :
             plfp ( array (crv.z).astype (Float), array (crv.y).astype (Float),
                    array (crv.x).astype (Float), array (crv.n).astype (Int),
                    hide = crv.hide )
          else :
             plfp ( crv.z , array (crv.y).astype (Float),
                    array (crv.x).astype (Float), array (crv.n).astype (Int),
                    hide = crv.hide )
       elif crv.type () == CellArrayType :
          if self._first_curve:
             self._first_curve = 0
             self.send_color_card ( )
             gridxy (self._gridtype)
             logxy ( self._xscale, self._yscale )
          if _color_bar_ :
             self.set_style ("z_work.gs")
             color_bar (min (ravel (crv.z)), max (ravel (crv.z)), 0)
          if crv.x0 is None and crv.x1 is None :
             pli (crv.z, hide = crv.hide )
          elif crv.x0 is None :
             pli (crv.z, crv.x1, crv.y1,
                  hide = crv.hide )
          else :
             pli (crv.z, crv.x0, crv.y0, crv.x1, crv.y1,
                  hide = crv.hide )
          self._plot_titles ( )
       elif crv.type () == QuadMeshType :
          if self._first_curve:
             self._first_curve = 0
             self.send_color_card ( )
             gridxy (self._gridtype)
             logxy ( self._xscale, self._yscale )
          self._plot_quadmesh (crv)
          self._plot_titles ( )
       elif crv.type () == Animation2dType :
          if self._first_curve:
             self._first_curve = 0
             self.send_color_card ( )
             gridxy (self._gridtype)
             logxy ( self._xscale, self._yscale )
             self._plot_titles ( )
          self._do_animation ( crv ) 
       else :
          raise self._GistError , \
             "Unknown object type '" + crv.type () + "'."

   _BadZScale = "BadZScale"

   def _do_animation ( self, anim ) :
       """_do_animation ( anim ) calls the functions supplied in
       object anim in order to do an animation.
       """
       # anim initializes its variables
       anim.initialize ( anim )

       if anim.animation == 1 :
          animate (1)
       else :
          animate (0)

       fma ( )
       for i in range (anim.nsteps) :
           for j in range (len (anim.calculations)) :
               anim.calculations [j] (anim)
               plg (anim.y, anim.x, color = anim.color)
           fma ( )
           anim.update (anim)

       if anim.animation == 1 :
           animate (0)
           for j in range (len (anim.calculations)) :
               anim.calculations [j] (anim)
               plg (anim.y, anim.x, color = anim.color)
          
       
   def _compute_levels ( self , crv , z) :
       """_compute_levels ( crv ) computes the z coordinates of levels
       of contours, if necessary. if crv.levels is an integer, it
       interpolates that many values between zmin and zmax, and
       puts those in crv.levels.
       """
       if type (crv.levels) == IntType and crv.levels >= 2 :
          if crv.z_scale == "normal" :
             zlin = ravel (z)
             lzlin = len (zlin)
             zbar = add.reduce (zlin) / lzlin
             zs = sqrt ( (add.reduce (zlin ** 2) - lzlin * zbar ** 2) /
                 (lzlin - 1))
             z1 = zbar - 2. * zs
             z2 = zbar + 2. * zs
             diff = (z2 - z1) / (crv.levels - 1)
             levs = z1 + arange (crv.levels) * diff
          elif crv.z_scale == "lin" or crv.z_scale == "log" :
             try: 
                [z1, z2] = zmin_zmax (z, crv.ireg)
             except:
                [z1, z2] = true_minmax (z)
          else :
             raise self._BadZScale , \
                `crv.z_scale` + " is an invalid z scale specifier."
          if crv.z_scale == "lin" :
             levs = z1 + arange (1, crv.levels, typecode = Float) * \
                (z2 - z1) / (crv.levels + 1)
          elif crv.z_scale == "log" :
             levs = z1 + exp (arange (1, crv.levels, typecode = Float) * \
                log (z2 - z1) / (crv.levels + 1))
          crv.levels = levs

   _QuadMeshError_ = "QuadMeshError"
 
   def _plot_quadmesh ( self , crv ) :
       """_plot_quadmesh ( <mesh object> ) is called internally by
       plot_object/add_object to plot meshes. This is because plotting
       a mesh has so many more options; this will keep add_object
       from getting way too long.
       """
       global _color_bar_
       if hasattr (crv, "regions") and crv.regions is not None and \
          crv.regions != [] and crv.regions != "all" :
             self._plot_by_regions ( crv )
             return
       if hasattr (crv, "regions") and crv.regions != "all" and \
          crv.regions != [] :
          raise self._QuadMeshError_, "<" + `crv.regions` + \
             "> is not a valid value for the regions keyword."
       if crv.z is not None and crv.filled == 0 and crv.contours == 1 :
          # Must be a contour plot
          if crv.boundary == 1 :
             plm ( boundary = 1, type = self._figure_type (crv.boundary_type) ,
                   color = self._figure_color (crv.boundary_color))
          if crv.levels is None :
             # Accept 8 default levels
             if crv.marks == 0 or crv.marker is None and crv.label != " " :
                marks = 1
                marker = crv.label [0]
             else :
                marks = crv.marks
                marker = crv.marker
             plc (array (crv.z).astype (Float),
                  hide = crv.hide, type = crv.line_type,
                  width = crv.width, color = crv.color, marks = marks,
                  marker = marker, region = crv.region)
          else :
             # Accept user-specified levels
             self._compute_levels (crv, crv.z)
             if crv.marks == 0 or crv.marker is None and crv.label != " " :
                marks = 1
                marker = crv.label [0]
             else :
                marks = crv.marks
                marker = crv.marker
             plc (array (crv.z).astype (Float),
                  hide = crv.hide, type = crv.line_type,
                  width = crv.width, color = crv.color, marks = marks,
                  marker = marker, region = crv.region, levs = crv.levels)
       elif crv.filled == 1 :
          # Must be a filled mesh or filled contours
          if _color_bar_ :
             self.set_style ("z_work.gs")
          if crv.contours == 0 :
             # Plot mesh filled with background color (wire frame)
             # (crv.z is None) or colored by crv.z
             if crv.levels is None :
                if _color_bar_ :
                   color_bar (min (ravel (crv.z)), max (ravel (crv.z)), 0, ncol = 8)
                plfc (crv.z, crv.y, crv.x, crv.ireg,
                     contours = 8,
                     region = crv.region, scale = crv.z_scale)
             else :
                if _color_bar_ :
                   color_bar (min (ravel (crv.z)), max (ravel (crv.z)), 0,
                      ncol=crv.levels)
                plfc (crv.z, crv.y, crv.x, crv.ireg,
                     contours = crv.levels,
                     region = crv.region, scale = crv.z_scale)
             return
          if self._cmax != 0.0 or self._cmin != 0.0 :
             # Use byte scale
             (k, l) = shape (crv.z)
             _z_ = reshape (bytscl (reshape (crv.z, (k * l,)),
                cmin = self._cmin, cmax = self._cmax), (k, l)).astype('b')
          else : 
             _z_ = crv.z
          if _color_bar_ :
             self.set_style ("z_work.gs")
          if crv.contours == 1 :
             # eventually this will plot filled contours
             if crv.marks == 0 or crv.marker is None and crv.label != " " :
                marks = 1
                marker = crv.label [0]
             else :
                marks = crv.marks
                marker = crv.marker
             if crv.levels is None :
                if _color_bar_ :
                   color_bar (min (ravel (_z_)), max (ravel (_z_)), 0, ncol = 8)
                plfc (_z_.astype (Float), crv.y.astype (Float),
                   crv.x.astype (Float), crv.ireg.astype (Int),
                   contours = 8, region = crv.region, 
                   scale = crv.z_scale)
                plc (crv.z.astype (Float),
                   hide = crv.hide, type = crv.line_type,
                   width = crv.width, color = crv.color, marks = marks,
                   marker = marker, region = crv.region)
             else :
                if _color_bar_ :
                   color_bar (min (ravel (_z_)), max (ravel (_z_)), 0,
                      ncol = crv.levels)
                plfc (_z_.astype (Float), crv.y.astype (Float),
                   crv.x.astype (Float), crv.ireg.astype (Int),
                   contours = crv.levels, region = crv.region, 
                   scale = crv.z_scale)
                self._compute_levels (crv, crv.z)
                plc (crv.z.astype (Float),
                   hide = crv.hide, type = crv.line_type, levs = crv.levels,
                   width = crv.width, color = crv.color, marks = marks,
                   marker = marker, region = crv.region)
          self._plot_titles ( )
       else :
          # Must be a plain, ordinary, every day, garden-variety mesh
          # If boundary = 1, we need to do a boundary plot.
          # if either ktype or ltype (or both) is not "none",
          # we need to plot the k lines or l lines (or both).
          if crv.boundary == 1 :
             plm (
                  boundary = 1, hide = crv.hide, 
                  type = self._figure_type (crv.boundary_type),
                  color = self._figure_color (crv.boundary_color), 
                  width = crv.width)#, region = crv.region)
          if crv.ktype != "none" or crv.ltype != "none" :
             if crv.ktype == crv.ltype :
                plm (
                     inhibit = crv.inhibit,
                     hide = crv.hide, type = crv.ktype,
                     width = crv.width, color = crv.color)#, region = crv.region)
                return
             if crv.ktype != "none" :
                plm (
                     inhibit = 2,
                     hide = crv.hide, type = crv.ktype,
                     width = crv.width, color = crv.color)#, region = crv.region)
             if crv.ltype != "none" :
                plm (
                     inhibit = 1,
                     hide = crv.hide, type = crv.ltype,
                     width = crv.width, color = crv.color)#, region = crv.region)
       if crv.vx is not None :
          # Must be a vector field plot on top of everything else
          if crv.scale is not None :
             plv (array (crv.vy).astype (Float), array (crv.vx).astype (Float),
                  hide = crv.hide,
                  width = crv.width, color = crv.color,
                  region = crv.region, scale = crv.scale)
          else :
             plv (array (crv.vy).astype (Float), array (crv.vx).astype (Float),
                  hide = crv.hide,
                  width = crv.width, color = crv.color,
                  region = crv.region)
 
   def _plot_by_regions ( self , crv ) :
       """
       _plot_by_regions (crv) allows one to plot some subset of the
       regions of a mesh with various differing options.
       """
       plmesh (array (crv.y).astype (Float), array (crv.x).astype (Float),
               array (crv.ireg).astype (Int),
               triangle = array (crv.tri).astype (Float))
       # Loop through the regions, plotting each as specified:
       for reg in crv.regions :
          if crv.z is not None and reg.filled == 0 and reg.contours == 1 :
             # Must be a contour plot
             if reg.boundary == 1 :
                plm ( boundary = 1,
                      type = self._figure_type (reg.boundary_type) ,
                      color = self._figure_color (reg.boundary_color))
             if reg.levels is None :
                # Accept 8 default levels
                if reg.marks == 0 or reg.marker is None and reg.label != " " :
                   marks = 1
                   marker = reg.label [0]
                else :
                   marks = reg.marks
                   marker = reg.marker
                plc (array (crv.z).astype (Float), 
                     hide = reg.hide, type = reg.line_type,
                     width = reg.width, color = self._figure_color (reg.color),
                     marks = marks, marker = marker,
                     region = reg.number)
             else :
                # Accept user-specified levels
                if reg.marks == 0 or reg.marker is None and reg.label != " " :
                   marks = 1
                   marker = reg.label [0]
                else :
                   marks = reg.marks
                   marker = reg.marker
                self._compute_levels (reg, crv.z)
                plc (array (crv.z).astype (Float), 
                     hide = reg.hide, type = reg.line_type,
                     width = reg.width, color = self._figure_color (reg.color),
                     marks = marks, marker = marker,
                     region = reg.number, levs = reg.levels)
          elif reg.filled == 1 :
             # Must be a filled mesh
             if crv.z is None :
                # Plot mesh filled with background color (wire frame)
                plf (crv.z, 
                   hide = reg.hide,region = reg.number,edges = reg.edges,
                   ecolor = self._figure_color (reg.ecolor),
                   ewidth = reg.ewidth)
                return
             if self._cmax != 0.0 or self._cmin != 0.0 :
                # Use byte scale
                # crv.z needs to be reshaped to linear then shaped
                # back to 2d.
                (k, l) = shape (crv.z)
                _z_ = reshape (bytscl (reshape (crv.z,  (k * l, )),
                   cmin = self._cmin, cmax = self._cmax), (k, l)).astype('b')
             else : 
                _z_ = crv.z
             plf (array (_z_).astype (Float), 
                  hide = reg.hide,
                  region = reg.number, edges = reg.edges, ecolor = 
                  self._figure_color (reg.ecolor),
                  ewidth = reg.ewidth)
          else :
             # Must be a plain, ordinary, every day, garden-variety mesh
             if reg.boundary == 1 :
                plm (boundary = 1,
                     hide = reg.hide, type = 
                     self._figure_type (reg.boundary_type),
                     width = reg.width,
                     color = self._figure_color (reg.boundary_color),
                     region = reg.number)
             if reg.ktype != "none" or reg.ltype != "none" :
                if reg.ktype == reg.ltype :
                   plm (inhibit = reg.inhibit, 
                        type = self._figure_type (reg.ktype),
                        width = reg.width,
                        color = self._figure_color (reg.color),
                        region = reg.number)
                else :
                   if reg.ktype != "none" :
                      plm (inhibit = 2,  
                           type = self._figure_type (reg.ktype),
                           width = reg.width,
                           color = self._figure_color (reg.color),
                           region = reg.number)
                   if reg.ltype != "none" :
                      plm (inhibit = 1,  
                           type = self._figure_type (reg.ltype),
                           width = reg.width,
                           color = self._figure_color (reg.color),
                           region = reg.number)
          if crv.vx is not None and reg.vectors != 0 :
             # Must be a vector field plot on top of anything else
             if ( crv.scale is not None ) :
                plv (array (crv.vy).astype (Float),
                     array (crv.vx).astype (Float),
                     scale = crv.scale,
                     hide = reg.hide,
                     width = reg.width, color = self._figure_color (reg.color),
                     region = reg.number)
             else :
                plv (array (crv.vy).astype (Float),
                     array (crv.vx).astype (Float),
                     hide = reg.hide,
                     width = reg.width, color = self._figure_color (reg.color),
                     region = reg.number)
 
   def send_graph ( self, graf ) :
       if graf.type () == Graph3dType :
          lims = draw3 (1)
          if lims is not None:
             limits (lims [0], lims [1], lims [2], lims [3])
          if hasattr (self, "_hcp") and self._hcp != "" :
             hcp ()
 
   def set_yr_axis_limits ( self , v, *w ) :
       pass
 
   def synchronize ( self ) :
       pass
 
   def set_label_type ( self, t ) :
       pass
 
   def set_y_axis ( self , lr , *lm ) :
       pass
 
   def set_freeze_each ( self , n ) :
       pass

   def set_connect (self, val) :
       pass

   def set_link (self, val) :
       pass

   def set_mask (self, val) :
       pass

   def set_z_c_switch (self, val) :
       self._z_c_switch = val

   def set_z_contours (self, val) :
       pass

   def set_c_contours (self, val) :
       pass

   def send_generics (self, graf) :
       """send_generics ( graf ) sets up all the things that are generic to any
       graph. It does not actually do any plotting yet.
       """
       self.set_titles ( graf._titles )
       self.set_title_colors ( graf._title_colors )
       # The following line is redundant for Gist. Not sure about Narcisse.
       # self._plot_titles ( )
       if self._dims == 2 :
          self.set_grid_type ( graf._grid_type )
       self.clear_text ( )
       if is_scalar ( graf._text ) :
          if ( graf._text != "" and graf._text != " ") :
             self.set_text ( graf._text , 0 )
       else :
          for i in range ( len ( graf._text ) ) :
             self.set_text ( graf._text [i] , i )
       if is_scalar ( graf._text_color ) :
          self.set_text_color ( graf._text_color , 0 )
       else :
          for i in range ( len ( graf._text_color ) ) :
             self.set_text_color ( graf._text_color [i] , i )
       if is_scalar ( graf._text_size ) :
          self.set_text_size ( graf._text_size , 0 )
       else :
          for i in range ( len ( graf._text_size ) ) :
              self.set_text_size ( graf._text_size [i] , i )
       if is_scalar ( graf._tosys ) :
          self.set_tosys ( graf._tosys , 0 )
       else :
          for i in range ( len ( graf._tosys ) ) :
              self.set_tosys ( graf._tosys [i] , i )
       if is_scalar ( graf._text_pos ) :
          raise graf._GraphSpecError , \
             "Text position must be a point or an array of points."
       if len ( shape ( graf._text_pos )) == 1:
          self.set_text_pos ( graf._text_pos [0] , graf._text_pos [1] , 0 )
       else :
          for i in range (shape (graf._text_pos) [0] ) :
             self.set_text_pos ( graf._text_pos [i][0] ,
                               graf._text_pos [i][1] , i )
       if hasattr (graf, "n") and self._dims == 3 :
          if graf.mask_change :
             self.set_mask ( graf._s [graf.n - 1].mask )
       try:
          no_color = os.environ["NO_COLOR"]
       except KeyError:
          no_color = 0
       if no_color == 0 or no_color == "no" or no_color == "n" :
          self.set_color_card ( graf._color_card )

   def do_generic (self, graf) :
       self.send_generics ( graf )
       self.equal_axes = 0
       self.set_x_axis_limits (graf._axis_limits [0][0],
                               graf._axis_limits [0][1])
       self.set_y_axis_limits (graf._axis_limits [1][0],
                               graf._axis_limits [1][1])
       if graf._no_of_axes > 2 :
          self.set_z_axis_limits (graf._axis_limits [2][0],
                                  graf._axis_limits [2][1])
       if graf._no_of_axes > 3 :
          self.set_c_axis_limits (graf._axis_limits [3][0],
                                  graf._axis_limits [3][1])
       for i in range (graf._no_of_axes) :
          if graf._axis_scales [i] == "lin" :
             self.set_axis_lin (graf._axes [i])
          elif graf._axis_scales [i] == "log" :
             self.set_axis_log (graf._axes [i])
          elif graf._axis_scales [i] == "equal" :
             self.equal_axes = 1
          else :
             raise graf._AxisSpecError , \
                graf._axis_scales [i] + " is not a valid axis scale."
       return

   def quick_plot (self, graf) :
       "quick_plot (graf) plots without recomputing."
       ## Note: I am using phi and theta as the standard polar
       ## angles, so I have put in conversions to the Gist model.
       if graf.type () == Graph2dType :
          self._dims = 2
       else :
          self._dims = 3
       self.do_generic (graf)
       if self._dims == 3 :
          phi = graf._theta
          theta = graf._phi
          if phi is not None:
             phi = pi / 2. - phi  * pi / 180.
          if theta is not None:
             theta = pi / 2. - theta * pi / 180.
          orient3 (phi = phi, theta = theta)
          light3 (ambient = graf.lighting_dict ["ambient"],
                  diffuse = graf.lighting_dict ["diffuse"],
                  specular = graf.lighting_dict ["specular"],
                  spower = graf.lighting_dict ["spower"],
                  sdir = graf.lighting_dict ["sdir"])
          lims = draw3 (1)
          if lims is not None:
             limits (lims [0], lims [1], lims [2], lims [3])
          if hasattr (self, "_hcp") and self._hcp != "" :
             hcp ()
          self._plot_titles ()
       return

   def plot2d (self, graf) :
       """A Graph2d object calls plot2d with itself as argument.
       plot2d sorts out everything for the graph and then does the plot.
       The bulk of this work used to be done in Graph and Graph2d,
       but I decided it was too graphics-dependent.
       """
       global _color_bar_
       _color_bar_ = graf._color_bar
       self._dims = 2
       # (1) Do graph-generic stuff first
       self.do_generic (graf)
       # (2) Do the specifically 2d stuff
       sc = graf._axis_scales
       if graf._xyequal :
          self.set_xyequal ( )
       else :
          self.reset_xyequal ( )
       if is_scalar (sc) :
          if sc == "linlin" or sc == "lin" :
             self.set_linlin ( )
          elif sc == "linlog" :
             self.set_linlog ( )
          elif sc == "loglin" or sc == "log" :
             self.set_loglin ( )
          elif sc == "loglog" :
             self.set_loglog ( )
       else :
          if len (sc) == 1 :
             sc = sc + ["lin", "lin"]
          elif len (sc) == 2 :
             sc = sc + ["lin"]
          for n in range (3) :
             if sc [n] == "log" :
                self.set_axis_log (graf._axes [n])
             else :
                self.set_axis_lin (graf._axes [n])
       for i in range ( graf._c_ln ) :
          if i == 0 :
             self.plot_object ( graf._c [i] )
             if self.equal_axes :
                maxv = max (max (graf._c [i].x), max (graf._c [i].y))
                minv = min (min (graf._c [i].x), min (graf._c [i].y))
          else :
             self.add_object ( graf._c [i] )
             if self.equal_axes :
                maxv = max (max (graf._c [i].x), max (graf._c [i].y), maxv)
                minv = min (min (graf._c [i].x), min (graf._c [i].y), minv)
       if self.equal_axes :
          limits (minv, maxv, minv, maxv)
       self.plot_text ( )

   def plot3d (self, graf, draw = 1) :
       """plot3d (graf) plots a 3d graph object.
       If draw = 0, it only does the calculations, then returns.
       """
       ## Note: I am using phi and theta as the standard polar
       ## angles, so I have put in conversions to the Gist model.
       self._dims = 3
       # (1) Do graph-generic stuff first
       ## For the moment I am ignoring all the generic stuff
       ## except for text and titles. Axis scales and limits
       ## are gilding the lily.
       self.do_generic (graf)

       set_draw3_ (0) # do not draw prematurely
       # 3d graph generics:
       self.set_distance (graf._distance)
       phi = graf._theta
       theta = graf._phi
       if phi is not None:
          phi = - pi / 2. - phi  * pi / 180.
       if theta is not None:
          theta = pi / 2. - theta * pi / 180.
       orient3 (phi = phi, theta = theta)
       if graf._gnomon is not None :
          gnomon (graf._gnomon, chr = [graf._axis_labels [0] [0],
                                       graf._axis_labels [1] [0],
                                       graf._axis_labels [2] [0]])
       light3 (ambient = graf.lighting_dict ["ambient"],
               diffuse = graf.lighting_dict ["diffuse"],
               specular = graf.lighting_dict ["specular"],
               spower = graf.lighting_dict ["spower"],
               sdir = graf.lighting_dict ["sdir"])
       if graf._x_factor != 1. or graf._y_factor != 1. :
          limits_ (xfactor = graf._x_factor, yfactor = graf._y_factor)
       else :
          limits_ (square = graf._xyequal) # set defaults
       if graf._s_ln == 1 and graf._s [0].type () == SurfaceType :
          # A single surface will be plotted by plwf. The
          # allowed options are any one of "wm", "f3", "f4", "s3";
          # or "wm" together with either "f3" or "f4". An
          # intelligent attempt will be made to convert other
          # specifications to one of these.
          #
          # If surface has a _color_card, let it overrule any set
          # by the graph to which it belongs.
          if (hasattr (graf._s [0], "_color_card")) :
             self.set_color_card (graf._s [0]._color_card)
          s = graf._s [0]
          z = s.z
          y = s.y
          x = s.x
          c = s.c
          quad_mesh = 0
          if x is not None and y is not None :
             if len (x.shape) == 2 and len (y.shape) == 2 :
                # Two dimensional quadrilateral mesh (ZCM 7/2/97)
                if x.shape [0] != y.shape [0] or x.shape [1] != y.shape [1] :
                   raise self._GistError, \
                      "shape of x '" + `x.shape` + "' and shape of y '" + \
                      `y.shape` + "' do not match."
             elif len (x.shape) != 1 or len (y.shape) != 1 :
                raise self._GistError, "x and y must be one dimensional."
             elif len (x) != shape (z) [0] or len (y) != shape (z) [1] :
                raise self._GistError, \
                   "lengths of x and y must match shape of z."
             else : # Both are 1; expand to 2d
                y = multiply.outer (ones (len (x), Float), y)
                x = multiply.outer (x, ones (len (y), Float))
          opt = s.opt_3d
          if is_scalar (opt) : opt = [opt]
          if "wm" in opt or "w3" in opt or "w4" in opt :
             edges = 1
          else :
             edges = 0
          c_color_bar = 0
          z_color_bar = 0
          if s.z_c_switch :
             z_axis_max = self._cmax
             z_axis_min = self._cmin
             c_axis_max = self._zmax
             c_axis_min = self._zmin
             dum = z
             z = c
             c = dum
             scale = s.c_contours_scale
             if s.c_contours_array is not None :
                contours = s.c_contours_array
                ncol = len (contours + 1)
             elif s.number_of_c_contours is not None :
                contours = s.number_of_c_contours
                ncol = contours + 1
             else :
                contours = 20
                ncol = contours + 1
          else :
             z_axis_max = self._zmax
             z_axis_min = self._zmin
             c_axis_max = self._cmax
             c_axis_min = self._cmin
             scale = s.z_contours_scale
             if s.z_contours_array is not None :
                contours = s.z_contours_array
                ncol = len (contours + 1)
             elif s.number_of_z_contours is not None :
                contours = s.number_of_z_contours
                ncol = contours + 1
             else :
                contours = 20
                ncol = contours + 1
          if ("f3" in opt or "s3" in opt or "w3" in opt or "i3" in opt) :
             fill = z
             z_color_bar = graf._color_bar
          elif ("f4" in opt or "s4" in opt or "w4" in opt or "i4" in opt) :
             fill = c
             c_color_bar = graf._color_bar
          else :
             fill = None
          if s.mask == "none" :
             cull = 0
          else :
             cull = 1
          if c_color_bar or z_color_bar :
             color_bar (min (ravel (fill)), max (ravel (fill)), graf._split, ncol)
          if "s3" in opt or "i3" in opt :
             # Plot contours in z (or c) direction
             if x is None and y is None :
                x = arange (fill.shape [0], typecode = Float)
                y = arange (fill.shape [1], typecode = Float)
             [nv, xyzv, dum] = slice3mesh (x, y, fill)
             if "s3" in opt :
                plzcont (nv, xyzv, contours = contours, scale = scale,
                   edges = edges, zaxis_min = z_axis_min, zaxis_max = z_axis_max)
             else :
                plzcont (nv, xyzv, contours = contours, scale = scale,
                   edges = edges, color = "bg", zaxis_min = z_axis_min,
                   zaxis_max = z_axis_max)
          elif "s4" in opt or "i4" in opt :
             # Plot contours in c (or z) direction
             if x is None and y is None :
                x = arange (fill.shape [0], typecode = Float)
                y = arange (fill.shape [1], typecode = Float)
             [nv, xyzv, col] = slice3mesh (x, y, z, color = fill, smooth = 1)
             if "s4" in opt :
                pl4cont (nv, xyzv, col, contours = contours, scale = scale,
                   edges = edges, caxis_min = c_axis_min, caxis_max = c_axis_max)
             else :
                pl4cont (nv, xyzv, col, contours = contours, scale = scale,
                   edges = edges, color = "bg", caxis_min = c_axis_min,
                   caxis_max = c_axis_max)
          else :
             plwf (z, y, x, fill = fill, shade = s.shade, edges = edges,
                ecolor = s.ecolor, ewidth = 1, cull = cull, scale =
                graf.z_scale)
          if c_color_bar or z_color_bar :
             self.set_style ("z_nobox.gs")
          else :
             self.set_style (graf._style)
          if draw == 0 :
             self.set_color_card (graf._color_card, now = 1)
             return
          self.lims = draw3 (1)
          if hasattr (self, "_hcp") and self._hcp != "" :
             hcp ()
          limits (self.lims [0], self.lims [1], self.lims [2], self.lims [3])
          self.send_color_card ()
       else :
          # Either there is more than one surface, or else at
          # least one is a mesh3d or a slice, so in any event must
          # be handled differently.  We have to accumulate the pieces
          # of the graph in a pl3tree.
          # N. B. I haven't implemented the Narcisse-style mesh3d
          # plot at this time. It is unclear to me how useful this is.
          clear3 ()
          isosurfaces_present = 0
          # If a split palette is called for, compute separate maxima and
          # minima for the whole graph (used to color plane sections)
          # and for isosurfaces (to effect maximum contrast in shading).
          zmax = None
          zmin = None
          for i in range (graf._s_ln) :
             s = graf._s [i]
             #
             # If some object has a _color_card, let it overrule any set
             # by the graph to which it belongs (last occurring will be used).
             if (hasattr (graf._s [i], "_color_card")) :
                self.set_color_card (graf._s [i]._color_card)
             if s.type () == Slice3dType :
                if s.plane is None :
                   isosurfaces_present = 1
                if s.val is not None : # Get max and min of values
                   if zmax is None :
                      zmax = max (s.val)
                      zmin = min (s.val)
                   else :
                      zmax = max (zmax, max (s.val))
                      zmin = min (zmin, min (s.val))
             elif s.type () == SurfaceType :
                isosurfaces_present = 1
          self.send_color_card ()
          if isosurfaces_present and graf._split != 0 :
             if self._current_palette is None:
                split_palette ( )
             else:
                split_palette (self._current_palette)
             spl = 1
          else:
             spl = 0
          for i in range (graf._s_ln) :
             ncol = None
             if i == 0 :
                clear3 ()
             s = graf._s [i]
             if s.type () == SurfaceType :
                z = s.z
                y = s.y
                x = s.x
                c = s.c
                opt = s.opt_3d
                if is_scalar (opt) : opt = [opt]
                if "wm" in opt :
                   edges = 1
                else :
                   edges = 0
                if s.z_c_switch :
                   z_axis_max = self._cmax
                   z_axis_min = self._cmin
                   c_axis_max = self._zmax
                   c_axis_min = self._zmin
                   dum = z
                   z = c
                   c = dum
                   scale = s.c_contours_scale
                   if s.c_contours_array is not None :
                      contours = s.c_contours_array
                   elif s.number_of_c_contours is not None :
                      contours = s.number_of_c_contours
                   else :
                      contours = 20
                else :
                   z_axis_max = self._zmax
                   z_axis_min = self._zmin
                   c_axis_max = self._cmax
                   c_axis_min = self._cmin
                   scale = s.z_contours_scale
                   if s.z_contours_array is not None :
                      contours = s.z_contours_array
                   elif s.number_of_z_contours is not None :
                      contours = s.number_of_z_contours
                   else :
                      contours = 20
                if ("f3" in opt or "s3" in opt or "w3" in opt) :
                   fill = z
                   z_color_bar = graf._color_bar
                elif ("f4" in opt or "s4" in opt or "w4" in opt) :
                   fill = c
                   c_color_bar = graf._color_bar
                else :
                   fill = None
                if "s3" in opt or "i3" in opt :
                   # Plot contours in z (or c) direction
                   if x is None and y is None :
                      x = arange (fill.shape [0], typecode = Float)
                      y = arange (fill.shape [1], typecode = Float)
                   [nv, xyzv, dum] = slice3mesh (x, y, fill)
                   if "s3" in opt :
                      plzcont (nv, xyzv, contours = contours, scale = scale,
                         clear = 0, edges = edges, zaxis_min = z_axis_min,
                         cmin = zmin, cmax = zmax, zaxis_max = z_axis_max)
                         # Do not clear multiple curves
                   else :
                      plzcont (nv, xyzv, contours = contours, scale = scale,
                         clear = 0, edges = edges, color = "bg", zaxis_min = z_axis_min,
                         cmin = zmin, cmax = zmax, 
                         zaxis_max = z_axis_max)
                elif "s4" in opt or "i4" in opt :
                   # Plot contours in c (or z) direction
                   if x is None and y is None :
                      x = arange (fill.shape [0], typecode = Float)
                      y = arange (fill.shape [1], typecode = Float)
                   [nv, xyzv, col] = slice3mesh (x, y, z, color = fill,
                       smooth = 1)
                   if "s4" in opt :
                      pl4cont (nv, xyzv, col, contours = contours,
                         scale = scale, clear = 0, edges = edges,
                         cmin = zmin, cmax = zmax,
                         caxis_min = c_axis_min, caxis_max = c_axis_max)
                   else :
                      pl4cont (nv, xyzv, col, contours = contours,
                         scale = scale, clear = 0, edges = edges, color = "bg",
                         cmin = zmin, cmax = zmax,
                         caxis_min = c_axis_min, caxis_max = c_axis_max)
                       # Do not clear multiple curves
                else :
                   if not hasattr (s, "s3") :
                      if x is None :
                         [nv, xyzv, val] = slice3mesh (z, color = c)
                      else :
                         [nv, xyzv, val] = slice3mesh (x, y, z, color = c)
                      setattr (s, "s3", Slice (nv, xyzv, val))
                   if zmax is None :
                      zmax = max (ravel (s.s3.val))
                      zmin = min (ravel (s.s3.val))
                   else :
                      zmax = max (zmax, max (ravel (s.s3.val)))
                      zmin = min (zmin, min (ravel (s.s3.val)))
                   pl3tree (s.s3.nv, s.s3.xyzv, s.s3.val, cmin = zmin, cmax = zmax,
                      split = spl, edges = edges)
             elif s.type () == Mesh3dType :
                raise self._GistError, "Narcisse style mesh plot not implemented."
             elif s.type () == Slice3dType :
                opt = s.opt_3d
                if type (opt) != ListType : opt = [opt]
                if "wm" in opt :
                   edges = 1
                else :
                   edges = 0
                # fval is what we plot if we ask for "flat" mode
                if "f3" in opt :
                   fval = s.xyzv[:,2]
                elif "f4" in opt :
                   fval = s.val
                if "s3" in opt or "i3" in opt or "s4" in opt or "i4" in opt :
                   if s.contours is None:
                      contours = 10
                      ncol = contours + 1
                   else :
                      contours = s.contours
                      if type (contours) == IntType :
                         ncol = contours + 1
                      else :
                         ncol = len (contours)
                   if "s3" in opt :
                      plzcont (s.nv, s.xyzv, contours = contours,
                         scale = s.scale, clear = 0, edges = s.edges,
                         cmin = zmin, cmax = zmax, zaxis_max = self._zmax,
                         zaxis_min = self._zmin, split = spl)
                   elif "i3" in opt :
                      plzcont (s.nv, s.xyzv, contours = contours,
                         scale = s.scale, clear = 0, edges = s.edges,
                         color = "bg", cmin = zmin, cmax = zmax,
                         split = spl)
                   elif "s4" in opt :
                      pl4cont (s.nv, s.xyzv, s.val, contours = contours,
                         scale = s.scale, clear = 0, edges = s.edges,
                         cmin = zmin, cmax = zmax,
                         caxis_min = self._cmin, caxis_max = self._cmax,
                         split = spl)
                   else :
                      pl4cont (s.nv, s.xyzv, s.val, contours = contours,
                         scale = s.scale, clear = 0, edges = s.edges,
                         cmin = zmin, cmax = zmax,
                         color = "bg", caxis_min = self._cmin, caxis_max = self._cmax,
                         split = spl)
                elif ("f3" in opt or "f4" in opt) and fval is not None \
                   and type (fval) == ArrayType :
                   if graf._split != 0 and isosurfaces_present :
                      if s.plane is not None :
                         pl3tree (s.nv, s.xyzv,
                            split_bytscl (fval, upper = 0, cmin = zmin,
                               cmax = zmax).astype('b'), s.plane,
                               cmin = zmin, cmax = zmax, split = 0,
                               edges = edges)
                      else :
                         pl3tree (s.nv, s.xyzv,
                            split_bytscl (fval, upper = 1, cmin = zmin,
                               cmax = zmax).astype('b'), s.plane,
                               cmin = zmin, cmax = zmax, split = 0,
                               edges = edges)
                   else :
                      pl3tree (s.nv, s.xyzv, bytscl (fval, top = 199,
                         cmin = zmin, cmax = zmax).astype('b'), s.plane,
                               cmin = zmin, cmax = zmax, split = 0,
                         edges = edges)
                else :
                   pl3tree (s.nv, s.xyzv, s.val, s.plane,
                         cmin = zmin, cmax = zmax, split = graf._split,
                         edges = edges)
             else :
                raise self._GistError, \
                   "object for 3d plot must be a Surface, a Mesh3d, " + \
                   "or one or more Slices."
          if zmax is not None and graf._color_bar != 0 :
             self.set_style ("z_nobox.gs")
          else :
             self.set_style (graf._style)
          if draw == 0 :
             return
          self.lims = draw3 (1)
          if hasattr (self, "_hcp") and self._hcp != "" :
             hcp ()
          if self.lims is not None :
             limits (self.lims [0], self.lims [1],
                self.lims [2], self.lims [3])
          if zmax is not None and zmin != zmax and graf._color_bar != 0 :
             color_bar (zmin, zmax, spl, ncol)

       # text must be done last or it might be obscured.
       self.plot_text ( )
       self._plot_titles ( )

   def move_light (self, i) :
       if i >= self.nframes : return 0
       theta = pi / 4. + (i - 1) * self.angle
       light3 (sdir = array ( [cos (theta), .25, sin(theta)], Float))
       draw3 (not self.making_movie)
       if hasattr (self, "_hcp") and self._hcp != "" :
          hcp ()
       return 1

   def move_light_source (self, graf, angle, nframes) :
       self.angle = angle
       self.nframes = nframes
       # Have to call plot3d to do calculations.
       self.plot3d (graf, draw = 0)
       limits (self.lims [0], self.lims [1], self.lims [2], self.lims [3])
       self.making_movie = 1
       set_draw3_ (0)
       movie (self.move_light, lims = self.lims)
       self.making_movie = 0
       fma ()
       self.move_light (1)

   def rotate_graph (self, graf, axis, angle, nframes) :
       # Have to call plot3d to do calculations.
       self.plot3d (graf, draw = 0)
       [lims0, lims1, lims2, lims3] = draw3 (1)
       limits (lims0, lims1, lims2, lims3)
       if hasattr (self, "_hcp") and self._hcp != "" :
          hcp ()
       spin3 (axis = axis, angle = angle * nframes,
          nframes = nframes) # , lims = [lims0, lims1, lims2, lims3])
       [lims0, lims1, lims2, lims3] = draw3 (1)
       limits (lims0, lims1, lims2, lims3)
       if hasattr (self, "_hcp") and self._hcp != "" :
          hcp ()




import re
import os
import sys

from cpuinfo import cpu
from fcompiler import FCompiler
from exec_command import exec_command, find_executable

class GnuFCompiler(FCompiler):

    compiler_type = 'gnu'
    version_pattern = r'GNU Fortran ((\(GCC[^\)]*(\)\)|\)))|)\s*'\
                      '(?P<version>[^\s*\)]+)'

    # 'g77 --version' results
    # SunOS: GNU Fortran (GCC 3.2) 3.2 20020814 (release)
    # Debian: GNU Fortran (GCC) 3.3.3 20040110 (prerelease) (Debian)
    #         GNU Fortran (GCC) 3.3.3 (Debian 20040401)
    #         GNU Fortran 0.5.25 20010319 (prerelease)
    # Redhat: GNU Fortran (GCC 3.2.2 20030222 (Red Hat Linux 3.2.2-5)) 3.2.2 20030222 (Red Hat Linux 3.2.2-5)

    for fc_exe in map(find_executable,['g77','f77']):
        if os.path.isfile(fc_exe):
            break
    executables = {
        'version_cmd'  : [fc_exe,"--version"],
        'compiler_f77' : [fc_exe,"-Wall","-fno-second-underscore"],
        'compiler_f90' : None,
        'compiler_fix' : None,
        'linker_so'    : [fc_exe],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"],
        }
    module_dir_switch = None
    module_include_switch = None

    # Cygwin: f771: warning: -fPIC ignored for target (all code is position independent)
    if os.name != 'nt' and sys.platform!='cygwin':
        pic_flags = ['-fPIC']

    #def get_linker_so(self):
    #    # win32 linking should be handled by standard linker
    #    # Darwin g77 cannot be used as a linker.
    #    #if re.match(r'(darwin)', sys.platform):
    #    #    return
    #    return FCompiler.get_linker_so(self)

    def get_flags_linker_so(self):
        opt = []
        if sys.platform=='darwin':
            if os.path.realpath(sys.executable).startswith('/System'):
                # This is when Python is from Apple framework
                opt.extend(["-Wl,-framework","-Wl,Python"])
            #else we are running in Fink python.
            opt.extend(["-lcc_dynamic","-bundle"])
        else:
            opt.append("-shared")
        if sys.platform[:5]=='sunos':
            # SunOS often has dynamically loaded symbols defined in the
            # static library libg2c.a  The linker doesn't like this.  To
            # ignore the problem, use the -mimpure-text flag.  It isn't
            # the safest thing, but seems to work. 'man gcc' says:
            # ".. Instead of using -mimpure-text, you should compile all
            #  source code with -fpic or -fPIC."
            opt.append('-mimpure-text')
        return opt

    def get_libgcc_dir(self):
        status, output = exec_command('%s -print-libgcc-file-name' \
                                      % (self.compiler_f77[0]),use_tee=0)        
        if not status:
            return os.path.dirname(output)
        return

    def get_library_dirs(self):
        opt = []
        if sys.platform[:5] != 'linux':
            d = self.get_libgcc_dir()
            if d:
                opt.append(d)
        return opt

    def get_libraries(self):
        opt = []
        d = self.get_libgcc_dir()
        if d is not None:
            for g2c in ['g2c-pic','g2c']:
                f = self.static_lib_format % (g2c, self.static_lib_extension)
                if os.path.isfile(os.path.join(d,f)):
                    break
        else:
            g2c = 'g2c'
        if sys.platform=='win32':
            opt.extend(['gcc',g2c])
        else:
            opt.append(g2c)
        return opt

    def get_flags_debug(self):
        return ['-g']

    def get_flags_opt(self):
        opt = ['-O3','-funroll-loops']
        return opt

    def get_flags_arch(self):
        opt = []
        if sys.platform=='darwin':
            if os.name != 'posix':
                # this should presumably correspond to Apple
                if cpu.is_ppc():
                    opt.append('-arch ppc')
                elif cpu.is_i386():
                    opt.append('-arch i386')
            for a in '601 602 603 603e 604 604e 620 630 740 7400 7450 750'\
                    '403 505 801 821 823 860'.split():
                if getattr(cpu,'is_ppc%s'%a)():
                    opt.append('-mcpu='+a)
                    opt.append('-mtune='+a)
                    break    
            return opt
        march_flag = 1
        # 0.5.25 corresponds to 2.95.x
        if self.get_version() == '0.5.26': # gcc 3.0
            if cpu.is_AthlonK6():
                opt.append('-march=k6')
            elif cpu.is_AthlonK7():
                opt.append('-march=athlon')
            else:
                march_flag = 0
        # Note: gcc 3.2 on win32 has breakage with -march specified
        elif self.get_version() >= '3.1.1' \
            and not sys.platform=='win32': # gcc >= 3.1.1
            if cpu.is_AthlonK6():
                opt.append('-march=k6')
            elif cpu.is_AthlonK6_2():
                opt.append('-march=k6-2')
            elif cpu.is_AthlonK6_3():
                opt.append('-march=k6-3')
            elif cpu.is_AthlonK7():
                opt.append('-march=athlon')
            elif cpu.is_AthlonMP():
                opt.append('-march=athlon-mp')
                # there's also: athlon-tbird, athlon-4, athlon-xp
            elif cpu.is_PentiumIV():
                opt.append('-march=pentium4')
            elif cpu.is_PentiumIII():
                opt.append('-march=pentium3')
            elif cpu.is_PentiumII():
                opt.append('-march=pentium2')
            else:
                march_flag = 0
            if self.get_version() >= '3.4' and not march_flag:
                march_flag = 1
                if cpu.is_Opteron():
                    opt.append('-march=opteron')
                elif cpu.is_Athlon64():
                    opt.append('-march=athlon64')
                else:
                    march_flag = 0
            if cpu.has_mmx(): opt.append('-mmmx')      
            if self.get_version() > '3.2.2':
                if cpu.has_sse2(): opt.append('-msse2')
                if cpu.has_sse(): opt.append('-msse')
            if self.get_version() >= '3.4':
                if cpu.has_sse3(): opt.append('-msse3')
            if cpu.has_3dnow(): opt.append('-m3dnow')
        else:
            march_flag = 0
        if march_flag:
            pass
        elif cpu.is_i686():
            opt.append('-march=i686')
        elif cpu.is_i586():
            opt.append('-march=i586')
        elif cpu.is_i486():
            opt.append('-march=i486')
        elif cpu.is_i386():
            opt.append('-march=i386')
        if cpu.is_Intel():
            opt.extend(['-malign-double','-fomit-frame-pointer'])
        return opt

if __name__ == '__main__':
    from scipy_distutils import log
    log.set_verbosity(2)
    from fcompiler import new_fcompiler
    #compiler = new_fcompiler(compiler='gnu')
    compiler = GnuFCompiler()
    compiler.customize()
    print compiler.get_version()



from wxPython import wx
import string
from Numeric import *
from scipy_base.fastumath import *
import time

from plot_utility import *
from plot_objects import *
from lena import lena

def loop():
    global bub
    bub.SetSize((400,400))
    for i in range(20):
        #if bub.GetSizeTuple()[0] ==400: bub.SetSize((200,200))
        #else: bub.SetSize((400,400))
        #bub.client.layout_all()
        bub.client.draw_graph_area()

# Issues:
#  -- Fix textobject.size() for non-90 degree rotations
#  -- Auto attributes haven't been fully thought through
#  -- Axis titles probably should be axis properties
#     This might save some current layout problems.
#  -- Little attention has been paid to round off errors.
#     Occassionly you'll see the consequences in a miss
#     placed grid line or slightly off markers, but overall
#     it's not so bad.
#  -- Could use a more sophisticated property setting scheme,
#     perhaps like graphite????  Anyway you can do bad things
#     such as assign a string instead of a text_object to
#     titles, etc.  This is bad.  Optional typing in future
#     Python would remove the need for fancy property type
#     checking system (I think).  Hope it comes to pass...
#  -- This should be split up into several modules.  Probably
#     a package.
#  -- Printing on windows does not print out line styles.  
#     Everything is printed as solid lines. argh!
#     Update: This seems like an issue with calling dc.SetUserScale
#     If this isn't called, the problem is fixed - but the graph is 
#     tiny!!!  Should I work out the appropriate scaling in draw()
#     or should the SetUserScale() method maintain line types?
#  -- Print Preview scaling is not correct.  Not sure why not,
#     but it looks lit the dc size is different the printer dc size.
#     How to fix?
#  -- Rotated text does not print with correct font type. Is this
#     a wxPython(windows) problem or mine?
#  -- Plot windows do not become top window when using gui_thread.

# To Add:
#  -- Legend
#  -- MouseDown support for changing fonts
from plot_utility import *



#make this a box_object?


aspect_ratios = ['normal', 'equal']

class plot_canvas(wx.wxWindow,property_object):
    _attributes = {
       'background_color': ['light grey',colors,"Window background color" \
                                           " Currently broken"],
       'aspect_ratio': ['normal',aspect_ratios,"Set the axis aspect ratio"],
       'hold': ['off',['on','off'],"Used externally for adding lines to plot"],
     }

    __type_hack__ = "plot_canvas"
    
    #background color is not working...
    def __init__(self, parent, id = -1, pos=wx.wxPyDefaultPosition,
                 size=wx.wxPyDefaultSize, **attr):
        wx.wxWindow.__init__(self, parent, id, pos,size)
        wx.EVT_PAINT(self,self.on_paint)
        property_object.__init__(self, attr)
        background = wx.wxNamedColour(self.background_color)
        self.SetBackgroundColour(background) 
        ##self.title = text_object('')
        ##self.x_title = text_object('')
        ##self.y_title = text_object('')
        self.title = text_window(self,'')
        self.x_title = text_window(self,'')
        self.y_title = text_window(self,'')
        self.all_titles = [self.title,self.x_title,self.y_title] #handy to have
        ##self.x_axis = axis_object(graph_location='above',rotate=0)        
        ##self.y_axis = axis_object(graph_location='right',rotate=90)
        self.x_axis = axis_window(self,graph_location='above',rotate=0)        
        self.y_axis = axis_window(self,graph_location='right',rotate=90)

        self.image_list = graphic_list()
        self.line_list = auto_line_list()  # make this the data object.
        self.legend = legend_object() 
        self.text_list = None  # list of text objects to place on screen
        self.overlays = None   # list of objects to draw on top of graph 
                               # (boxes, circles, etc.)
        ##self.y2_axis = axis_object(graph_location='left',rotate=90) 
        self.client_size = (0,0)
        # zoom selection helpers
        self._mouse_selection = 0
        self._mouse_selection_start = wx.wxPoint(0,0)
        self._mouse_selection_stop = wx.wxPoint(0,0)
        # mouse events
        wx.EVT_RIGHT_DOWN(self,self.on_right_down)
        wx.EVT_LEFT_DOWN(self, self.on_mouse_event)
        wx.EVT_LEFT_UP(self, self.on_mouse_event)
        wx.EVT_MOTION(self, self.on_mouse_event)
        wx.EVT_MOTION(self, self.on_mouse_event)
        wx.EVT_SIZE(self, self.update)

    # event handler

    def on_mouse_event(self, event):
        if event.LeftDown():
            # start zoom selection
            self._mouse_selection_start = event.GetPosition()
            self._mouse_selection_stop = event.GetPosition()
            self._mouse_selection = 1
        elif event.LeftUp() and self._mouse_selection:
            # zoom in
            self.zoom(event.GetPosition())
            self._mouse_selection = 0
        elif event.LeftIsDown() and self._mouse_selection:
            # update zoom selection
            self.rubberband(event.GetPosition())
        else:
            pass

    def auto_zoom(self):
        # zoom to full scale
        # cannot call autoscale(), since validate_active() apparently
        # leads to a race condition:(
        self.x_axis.bounds = ['auto','auto']
        self.y_axis.bounds = ['auto','auto']
        self.x_axis.tick_interval = 'auto'
        self.y_axis.tick_interval = 'auto'
        self.update()
    
    def on_auto_zoom(self,event):
        self.auto_zoom()    
        
    def on_paint(self, event):
        self.draw(wx.wxPaintDC(self))

    def on_right_down(self,event):
        menu_found = 0
        pos = event.GetX(),event.GetY()
        dc = wx.wxClientDC(self)
        for title in self.all_titles:
            title.set_dc(dc) # this dc stuff is a pain...
            if title.contains(pos):
                title.format_popup(pos)
                break
                menu_found = 1
            title.clear_dc()
        if not menu_found:
            if self.x_axis.contains(pos,dc):
                self.x_axis.format_popup(pos)
            elif self.y_axis.contains(pos,dc):
                self.y_axis.format_popup(pos)
            else:
                self.format_popup(pos)

    def format_popup(self,pos):
        menu = wx.wxMenu()
        menu.Append(500, 'Auto Zoom', 'Auto Zoom')
        wx.EVT_MENU(self, 500, self.on_auto_zoom)
        menu.UpdateUI()
        self.PopupMenuXY(menu,pos[0],pos[1])        
    # workers
    
    def rubberband(self, new):
        """Delete previous selection band and paint new one."""
        if self._mouse_selection:
            dc = wx.wxClientDC(self)
            dc.SetLogicalFunction(wx.wxXOR)
            dc.SetPen(wx.wxGREY_PEN)
            dc.SetBrush(wx.wxTRANSPARENT_BRUSH)
            diff = self._mouse_selection_stop - self._mouse_selection_start
            dc.DrawRectangle(self._mouse_selection_start.x,
                             self._mouse_selection_start.y, diff.x, diff.y)
            self._mouse_selection_stop = new
            diff = self._mouse_selection_stop - self._mouse_selection_start
            dc.DrawRectangle(self._mouse_selection_start.x,
                             self._mouse_selection_start.y, diff.x, diff.y)

    def save(self,path,image_type):
        w,h = self.GetSizeTuple()
        bitmap = wx.wxEmptyBitmap(w,h)
        dc = wx.wxMemoryDC()
        dc.SelectObject(bitmap)
        #self.update()
        # The background isn't drawn right without this cluge.   
        #fill_color = get_color(self.background_color)
        fill_color = get_color('white')        
        dc.SetPen(wx.wxPen(fill_color))
        dc.SetBrush(wx.wxBrush(fill_color)) #how to handle transparency???
        dc.DrawRectangle(0,0,w,h)
        dc.SetPen(wx.wxNullPen)
        dc.SetBrush(wx.wxNullBrush)
        # end cluge
        self.draw(dc)
        image = wx.wxImageFromBitmap(bitmap)
        wx.wxInitAllImageHandlers()
        image.SaveFile(path,image_type_map[image_type])

    def layout_all(self,dc=None):
        #settingbackgroundcolors
        #background = wx.wxNamedColour(self.background_color)
        #if self.GetBackgroundColour() != background:
        #   self.SetBackgroundColour(background) 
           #self.Clear()
        #   print 'refreshing'  
        if not dc: dc = wx.wxClientDC(self)            
        self.client_size = array(self.GetClientSizeTuple())
        # set the device context for all titles so they can
        # calculate their size
        for text_obj in self.all_titles:
            text_obj.set_dc(dc)
            
        graph_area = box_object((0,0),self.client_size)
        graph_area.inflate(.95) # shrink box slightly
        
        # shrink graph area to make room for titles
        graph_area = self.layout_border_text(graph_area)        
        # layout axis and graph data
        graph_area = self.layout_graph(graph_area,dc)
        # center titles around graph area.
        self.finalize_border_text(graph_area,dc)   
        self.graph_box = graph_area
        # clear the dc for all titles
        # ? neccessary ?
        for text_obj in self.all_titles:
            text_obj.clear_dc()
        self.layout_data()
        
        #self.legend.layout(self.line_list,graph_area,dc)
        
    def layout_border_text(self,graph_area):
        # Shrink graph area to make room for titles.
        # Also, specify where the text is to live
        # in realation to the graph.  This only
        # specifies one axis.  The other can only
        # be specified after the final graph area
        # is calculated.
        margin = 4
        graph_area.trim_top(self.title.height()+margin)
        graph_area.trim_bottom(self.x_title.height()+margin)            
        self.y_title.rotate = 90 # make sure it is rotated
        graph_area.trim_left(self.y_title.width()+margin)
        #this is just to make so extra room for axis labels
        #on the x axis...
        graph_area.trim_right(12)
        return graph_area

    def layout_data(self):    
        # get scale and offset
        axis_range = array((self.x_axis.range(),self.y_axis.range()),Float)
        # negative y to account for positve down in window coordinates
        scale = self.graph_box.size() / axis_range * array((1.,-1.))
        offset = self.graph_to_window(array((0.,0.)))
        self.image_list.scale_and_shift(scale,offset)
        self.line_list.scale_and_shift(scale,offset)        
        #self.legend 
        #self.text_list
        #self.overlays
        
    def layout_graph(self,graph_area,dc):                        
        self.axes = []
        #data_x_bounds,data_y_bounds = [0,6.28], [-1.1,1000]
        #jeez this is unwieldy code...
        smalls = []; bigs =[]
        if len(self.line_list):
            p1,p2 =  self.line_list.bounding_box()
            smalls.append(p1);bigs.append(p2)
        if len(self.image_list):
            p1,p2 =  self.image_list.bounding_box()
            smalls.append(p1);bigs.append(p2)        
        if len(smalls):    
            min_point = minimum.reduce(smalls)
            max_point = maximum.reduce(bigs)
        else:
            min_point = array((-1.,-1.),)
            max_point = array((1.,1.))               
        data_x_bounds = array((min_point[0],max_point[0]))
        data_y_bounds = array((min_point[1],max_point[1]))
        self.x_axis.calculate_ticks(data_x_bounds)
        height = self.x_axis.max_label_height(dc)
        graph_area.trim_bottom(height)
        self.y_axis.calculate_ticks(data_y_bounds)
        width = self.y_axis.max_label_width(dc)
        graph_area.trim_left(width)
        if self.aspect_ratio == 'equal':
            x_scale = float(graph_area.width()) / self.x_axis.range()
            y_scale = float(graph_area.height()) / self.y_axis.range()
            #print 'scales:', x_scale,y_scale,self.x_axis.range(),self.y_axis.range()
            if x_scale > y_scale:
                new_width = y_scale * self.x_axis.range()
                remove = .5 * (graph_area.width() - new_width)
                graph_area.trim_left(remove)
                graph_area.trim_right(remove)
            else:    
                new_height = x_scale * self.y_axis.range()
                remove = .5 * (graph_area.height() - new_height)
                graph_area.trim_top(remove)
                graph_area.trim_bottom(remove)
        #self.y2_axis = axis_object(graph_location='left',rotate=90)
        #self.y2_axis.label_location = 'plus'
        #self.y2_axis.calculate_ticks(y2bounds)
        #width = self.y2_axis.max_label_width(dc)
        #graph_area.trim_right(width)
        self.x_axis.layout(graph_area,dc)
        self.x_axis.move((graph_area.left(),graph_area.bottom()))
        self.axes.append(self.x_axis)
        self.y_axis.layout(graph_area,dc)
        self.y_axis.move((graph_area.left(),graph_area.bottom()))
        self.axes.append(self.y_axis)
        #self.y2_axis.grid_color = 'wheat'
        #self.y2_axis.layout(graph_area,dc)
        #self.y2_axis.move((graph_area.right(),graph_area.bottom()))
        #self.axes.append(self.y2_axis)
        self.border = border_object()
        self.border.layout(graph_area,self.x_axis,self.y_axis)
        return graph_area
        
    def finalize_border_text(self,graph_area,dc):
        # Center the titles around the graph.
        # -- Really need to make axis object box_objects.
        #    Use this to help determine more appropriate 
        #    title location.  Current works fine
        #    if axis labels are beside graph.  Title
        #    will be to far away if they are in center of graph        
        margin = 4
        if self.title:   
            self.title.center_on_x_of(graph_area)
            self.title.above(graph_area,margin)
        if self.x_title: 
            offset = self.x_axis.max_label_height(dc)
            self.x_title.center_on_x_of(graph_area)
            self.x_title.below(graph_area,margin + offset)
        if self.y_title: 
            offset = self.y_axis.max_label_width(dc)
            self.y_title.center_on_y_of(graph_area)
            self.y_title.left_of(graph_area,margin+offset)
        #if self.y2_title:self.y2_title.center_on_y_of(graph_area)
    
    def graph_to_window(self,pts):
        axis_range =  array((self.x_axis.range(),self.y_axis.range()))
        # negative y to account for positve down in window coordinates
        scale = self.graph_box.size() / axis_range * array((1.,-1.))
        graph_min = array((self.x_axis.ticks[0],self.y_axis.ticks[0]))
        zero_offset = (array((0.,0))- graph_min)  * scale 
        graph_offset = array((self.graph_box.left(),self.graph_box.bottom()))
        return pts * scale + zero_offset + graph_offset
                   
    def reset_size(self, dc = None):
        new_size = self.GetClientSizeTuple()
        if new_size != self.client_size:
            self.layout_all(dc)
            self.client_size = new_size

    def draw_graph_area(self,dc=None):
        if not dc: dc = wx.wxClientDC(self)                                     
        self.layout_data() # just to check how real time plot would go...

        gb = self.graph_box
        #clear the plot area
        # SHOULD SET PEN HERE TO FILL BACKGROUND WITH CORRECT COLOR
        fill_color = get_color('white')
        dc.SetPen(wx.wxPen(fill_color))
        dc.SetBrush(wx.wxBrush(fill_color))
        # NEEDED FOR REAL-TIME PLOTTING
        dc.DrawRectangle(gb.left(),gb.top(),
                         gb.width()+1,gb.height()+1)
        #needed to make sure images stay within bounds
        ##dc.SetClippingRegion(gb.left()-1,gb.top()-1,
        ##                     gb.width()+2,gb.height()+2)  # mod by GAP 26092003
        dc.SetClippingRegion(int(gb.left()-1),int(gb.top()-1),
                             int(gb.width()+2),int(gb.height()+2))
        # draw images
        self.image_list.draw(dc)
        dc.DestroyClippingRegion()        
        # draw axes lines and tick marks               
        t1 = time.clock()    
        for axis in self.axes:
            axis.draw_lines(dc)
        #for axis in self.axes:
        #    axis.draw_grid_lines(dc)
        #for axis in self.axes:
        #    axis.draw_ticks(dc)    
        t2 = time.clock()
        #print 'lines:', t2 - t1
        #draw border
        t1 = time.clock(); self.border.draw(dc); t2 = time.clock()
        #print 'border:', t2 - t1                    
        # slightly larger clipping area so that marks
        # aren't clipped on edges
        # should really clip markers and lines separately
        # draw lines
        self.line_list.clip_box(self.graph_box)
        self.line_list.draw(dc)
        # draw text        
        # draw legend
        # self.legend.draw(dc)
        # draw overlay objects
        
    def draw(self,dc=None):
        #if not len(self.line_list) or len(self.image_list):
        #    return
        # resize if necessary
        #print 'draw'
        #print 'dc:',dc
        t1 = time.clock();self.reset_size(dc);t2 = time.clock()
        #print 'resize:',t2 - t1        
        if not dc: dc = wx.wxClientDC(self)
        # draw titles and axes labels
        t1 = time.clock()    
        for text in self.all_titles:
            text.draw(dc)        
        for axis in self.axes:
            axis.draw_labels(dc)
        t2 = time.clock()
        #print 'text:',t2 - t1
        self.draw_graph_area(dc)
            
    def update(self,event=None):
        self.client_size = (0,0) # forces the layout
        self.Refresh()        

    def zoom(self, stop):
        """Delete selection band and zoom selection to full scale."""
        # delete rubberband
        dc = wx.wxClientDC(self)
        dc.SetLogicalFunction(wx.wxXOR)
        dc.SetPen(wx.wxGREY_PEN)
        dc.SetBrush(wx.wxTRANSPARENT_BRUSH)
        diff = self._mouse_selection_stop - self._mouse_selection_start
        dc.DrawRectangle(self._mouse_selection_start.x,
                         self._mouse_selection_start.y, diff.x, diff.y)
        self._mouse_selection_stop = stop
        # get zoom-area coordinates
        p1 = self._mouse_selection_start
        p2 = self._mouse_selection_stop
        gb = self.graph_box
        if abs(p1.x-p2.x) < 3 and abs(p1.y-p2.y) < 3:
            # zoom aborted (we assume)
            return
        left = float(min(p1.x, p2.x) - gb.left()) / gb.width()
        right = float(max(p1.x, p2.x) - gb.left()) / gb.width()
        top = float(min(p1.y, p2.y) - gb.top()) / gb.height()
        bottom = float(max(p1.y, p2.y) - gb.top()) / gb.height()
        # convert to real bounds
        width = self.x_axis.ticks[-1] - self.x_axis.ticks[0]
        height = self.y_axis.ticks[-1] - self.y_axis.ticks[0]
        left = left * width + self.x_axis.ticks[0]
        right = right * width + self.x_axis.ticks[0]
        top = self.y_axis.ticks[-1] - top * height
        bottom = self.y_axis.ticks[-1] - bottom * height
        x_int = auto_interval([left, right])
        y_int = auto_interval([bottom,top])
        self.x_axis.bounds = auto_bounds([left, right],x_int)
        self.y_axis.bounds = auto_bounds([bottom, top],y_int)
        self.update()




#------------------ tick utilities -----------------------
# flexible log function
#------------------ end tick utilities -----------------------

class graph_printout(wx.wxPrintout):
    """Print wrapper."""
    # Do not change method names in this class,
    # we have to override wxPrintout methods here!
    def __init__(self, graph):
        wx.wxPrintout.__init__(self)
        self.graph = graph

    def HasPage(self, page):
        if page == 1:
            return wx.true
        else:
            return wx.false

    def GetPageInfo(self):
        return (1, 1, 1, 1)

    def OnPrintPage(self, page):
        dc = self.GetDC()
        # .5 inch margins are automatic
        # on my HP.  Probably not standard.
        # Need smarter margin control.
        w_inch,h_inch = self.GetPPIPrinter()
        x_margin = .0* w_inch
        y_margin = .0* h_inch
        #-------------------------------------------
        # One possible method of setting scaling factors...
        #print w_inch,h_inch
        #print dc.GetSizeTuple()        
        graph_box = box_object((0,0),self.graph.GetSizeTuple())
        # Get the size of the DC in pixels
        page_size = dc.GetSizeTuple()
        #print 'dc size:',page_size
        #page_size = self.GetPageSizePixels()
        #print 'page size:',page_size
        print_box = box_object((0,0),page_size)        
        print_box.trim_left(x_margin)
        print_box.trim_right(x_margin)
        print_box.trim_top(y_margin)
        print_box.trim_bottom(y_margin)
        # Calculate a suitable scaling factor
        scales = array(print_box.size(), Float)/graph_box.size()
        # Use x or y scaling factor, whichever fits on the DC
        scale = min(scales)
        # resize the graph and center on the page
        graph_box.inflate(scale)
        graph_box.center_on(print_box)
        # set the device scale and origin
        dc.SetUserScale(scale, scale)
        dc.SetDeviceOrigin(graph_box.left(),graph_box.top())
        #-------------------------------------------
        #print 'print dc size:', dc.GetSizeTuple()
        self.graph.draw(dc)
        return wx.true


class plot_window(plot_canvas):
    """Plot canvas window.

    This is a complete q+d hack, but it does (somewhat) work.
    """
    def __init__(self, parent, id = -1, pos=wx.wxPyDefaultPosition,
                 size=wx.wxPyDefaultSize,**attr):
        plot_canvas.__init__(self, parent, id, pos, size)
        self.parent = parent
        self.proxy_object_alive = wx.true
        scipy.plt.figure(self)
        self.layout_all()
        scipy.plt.xaxis('fit')
        # initial print setup
        self.print_data = wx.wxPrintData()
        self.print_data.SetPaperId(wx.wxPAPER_LETTER)
        self.print_data.SetOrientation(wx.wxLANDSCAPE)

    def add(self, x, y, **keywds):
        """Add data to plot."""
        groups = scipy.plt.plot_groups([x, y, 'b-'])
        lines = []
        for group in groups:
            lines.extend(scipy.plt.lines_from_group(group))
        # check for hold here
        for name in plot_objects.poly_marker._attributes.keys():
            value = keywds.get(name)
            if value is not None:
                for k in range(len(lines)):
                    exec('lines[k].markers.%s = value' % name)
        for i in lines:
            self.line_list.append(i)
        self.update();

    def clear(self):
        """Clear canvas, remove all plots."""
        self.line_list = auto_line_list()
        scipy.plt.autoscale()

    def set_title(self, title):
        """Set plot title."""
        self.title = text_window(self, title)
        self.all_titles[0] = self.title
        self.update()
        
    def set_xaxis_title(self, xtitle):
        """Set x-axis title"""
        self.x_title = text_window(self, xtitle)
        self.all_titles[1] = self.x_title
        self.update()

    def set_yaxis_title(self, ytitle):
        """Set y-axis title"""
        self.y_title = text_window(self, ytitle)
        self.all_titles[2] = self.y_title
        self.update()

    def printout(self, paper=None):
        """Print current plot."""
        if paper != None:
            self.print_data.SetPaperId(paper)
        pdd = wx.wxPrintDialogData()
        pdd.SetPrintData(self.print_data)
        printer = wx.wxPrinter(pdd)
        out = graph_printout(self)
        print_ok = printer.Print(self.parent, out)
        if print_ok:
            self.print_data = printer.GetPrintDialogData().GetPrintData()
        out.Destroy()

    def print_preview(self):
        """Print-preview current plot."""
        printout = graph_printout(self)
        printout2 = graph_printout(self)
        self.preview = wx.wxPrintPreview(printout, printout2, self.print_data)
        if not self.preview.Ok():
            wx.wxMessageDialog(self, "Print Preview failed.\n" \
                               "Check that default printer is configured\n", \
                               "Print error", wx.wxOK|wx.wxCENTRE).ShowModal()
        else:
            frame = wx.wxPreviewFrame(self.preview, self.parent, "Preview")
            frame.Initialize()
            frame.SetPosition(self.GetPosition())
            frame.SetSize(self.GetSize())
            frame.Show(wx.true)


class plot_frame(wx.wxFrame):
    """wxFrame for interactive use of plot_canvas."""
    TITLE_FONT = 210
    AXIS_FONT = 211
    LABEL_FONT = 212
    
    TITLE_TEXT,X_TEXT,Y_TEXT = 220,221,222

    default_size = (500,400) # the default on Linux is always tiny???
    
    def __init__(self, parent=wx.NULL, id = -1, title = '', 
                 pos=wx.wxPyDefaultPosition,
                 size=default_size,visible=1):
        wx.wxFrame.__init__(self, parent, id, title,pos,size)
        # Now Create the menu bar and items
        self.mainmenu = wx.wxMenuBar()
        menu = wx.wxMenu()
        menu.Append(200, '&Save As...', 'Save plot to image file')
        wx.EVT_MENU(self, 200, self.file_save_as)
        menu.Append(203, '&Print...', 'Print the current plot')
        wx.EVT_MENU(self, 203, self.file_print)
        menu.Append(204, 'Print Pre&view', 'Preview the current plot')
        wx.EVT_MENU(self, 204, self.file_preview)
        menu.Append(205, 'Close', 'Close plot')
        wx.EVT_MENU(self, 205, self.file_close)
        self.mainmenu.Append(menu, '&File')
        menu = wx.wxMenu()
        menu.Append(self.TITLE_TEXT, '&Graph Title', 'Title for plot')
        wx.EVT_MENU(self,self.TITLE_TEXT,self.title)
        menu.Append(self.X_TEXT, '&X Title', 'Title for X axis')
        wx.EVT_MENU(self,self.X_TEXT,self.title)
        menu.Append(self.Y_TEXT, '&Y Title', 'Title for Y axis')
        wx.EVT_MENU(self,self.Y_TEXT,self.title)
        self.mainmenu.Append(menu, '&Titles')
        #menu = wx.wxMenu()        
        #menu.Append(300, '&Profile', 'Check the hot spots in the program')
        #wx.EVT_MENU(self,300,self.OnProfile)
        #self.mainmenu.Append(menu, '&Utility')
        self.SetMenuBar(self.mainmenu)
        # A status bar to tell people what's happening
        self.CreateStatusBar(1)
        self.print_data = wx.wxPrintData()
        self.print_data.SetPaperId(wx.wxPAPER_LETTER)
        self.client = plot_canvas(self)
        if visible: self.Show(1)
        self.Raise()
        self.SetFocus()        
                    
    def plot_draw(self, event):
        #self.client.graphics = _InitObjects()
        self.client.title.text = 'Bubba'
        self.client.x_title.text = 'x title'
        self.client.y_title.text = 'y title'
        #self.client.y2_title.text = 'y2 title'
        for i in _InitObjects():
            self.client.line_list.append(i)
        #self.client.image_list.append(lena_obj())    
        self.client.draw();


    def profile(self, event):
        import profile
        #self.client.graphics = _InitObjects()
        self.client.title.text = 'Bubba'
        self.client.x_title.text = 'x title'
        self.client.y_title.text = 'y title'
        #self.client.y2_title.text = 'y2 title'
        #for i in _InitObjects():
        #    self.client.line_list.append(i)
        #self.client.image_list.append(lena_obj())    
        global bub
        bub = self
        profile.run('from plt import loop;loop()','profile')        
        
    def file_print(self, event):
        self.print_data.SetPaperId(wx.wxPAPER_LETTER)
        pdd = wx.wxPrintDialogData()
        pdd.SetPrintData(self.print_data)
        printer = wx.wxPrinter(pdd)
        printout = graph_printout(self.client)
        print_ok = printer.Print(self, printout)
        #Is Abort() not wrapped?
        #if not printer.Abort() and not print_ok:     
        #    wx.wxMessageBox("There was a problem printing.\n" \
        #                    "Perhaps your current printer is not set correctly?",
        #                    "Printing", wx.wxOK)
        #else:
        #    self.print_data = printer.GetPrintDialogData().GetPrintData()
        if print_ok:
            self.print_data = printer.GetPrintDialogData().GetPrintData()
        printout.Destroy()

    def file_preview(self, event):
        printout = graph_printout(self.client)
        printout2 = graph_printout(self.client)
        self.preview = wx.wxPrintPreview(printout, printout2, self.print_data)
        if not self.preview.Ok():
            #self.log.WriteText("Print Preview failed." \
            #                   "Check that default printer is configured\n")
            print "Print Preview failed." \
                  "Check that default printer is configured\n"
            return
        frame = wx.wxPreviewFrame(self.preview, self, "Preview")
        frame.Initialize()
        frame.SetPosition(self.GetPosition())
        frame.SetSize(self.GetSize())
        frame.Show(wx.true)

    def file_save_as(self, event):
        import os
        wildcard = "PNG files (*.png)|*.png|" \
                   "BMP files (*.bmp)|*.bmp|" \
                   "JPEG files (*.jpg)|*.jpg|" \
                   "PCX files (*.pcx)|*.pcx|" \
                   "TIFF files (*.tif)|*.tif|" \
                   "All Files |*|"
        dlg = wx.wxFileDialog(self, "Save As", ".", "", wildcard, wx.wxSAVE)
        if dlg.ShowModal() == wx.wxID_OK:
            f = dlg.GetPath()
            dummy, ftype = os.path.splitext(f)
            # strip .
            ftype = ftype[1:]
            if ftype in image_type_map.keys():
                self.client.save(dlg.GetPath(),ftype)
            else:
                msg = "Extension is currently used to determine file type." \
                      "'%s' is not a valid extension."  \
                      "You may use one of the following extensions. %s" \
                          % (ftype,image_type_map.keys())   
                d = wx.wxMessageDialog(self,msg,style=wx.wxOK)
                d.ShowModal()
                d.Destroy()
        dlg.Destroy()

    def file_close(self, event):
        self.Close()
        
    def format_font(self,event):
        font_attr,color_attr = 'font','color'
        if event.GetId() == self.TITLE_FONT:
            texts = [self.client.title]
        elif event.GetId() == self.AXIS_FONT:            
            texts = [self.client.x_title,self.client.y_title]
           #texts = [self.client.y_title]
        elif event.GetId() == self.LABEL_FONT:
            texts = [self.client.x_axis,self.client.y_axis]
            font_attr,color_attr = 'label_font','label_color'
        data = wx.wxFontData()
        current_color = get_color(getattr(texts[0],color_attr))
        current_font = getattr(texts[0],font_attr)
        data.SetColour(current_color)
        data.SetInitialFont(current_font)
        dlg = wx.wxFontDialog(self, data)
        if dlg.ShowModal() == wx.wxID_OK:
            data = dlg.GetFontData()
            font = data.GetChosenFont()
            color = data.GetColour()
            rgb = color.Red(),color.Green(),color.Blue()
            for text in texts:
                setattr(text,color_attr,rgb)
                setattr(text,font_attr,font)
                self.client.update()
        dlg.Destroy()

    def title(self,event):
        if event.GetId() == self.TITLE_TEXT:
            title = self.client.title
            prompt = 'Enter graph title'
        elif event.GetId() == self.X_TEXT:            
            title = self.client.x_title
            prompt = 'Enter x axis title'
        elif event.GetId() == self.Y_TEXT:
            title = self.client.y_title
            prompt = 'Enter y axis title'        
        dlg = wx.wxTextEntryDialog(self, prompt,'', title.text)
        if dlg.ShowModal() == wx.wxID_OK:
            title.text = dlg.GetValue()
        dlg.Destroy()
        self.client.update()

    def update(self):
        self.client.update()
            
    def __getattr__(self,key):
        try:        
            return self.__dict__[key]
        except KeyError:  
            return getattr(self.__dict__['client'],key)
    """        
    def __setattr__(self,key,val):
        #print key,val
        #if plot_canvas._attributes.has_key(key):
        #    self.__dict__['client'].__dict__[key] = val
        #    return None
        self.__dict__[key] = val
    """    


# global functions

def lena_obj():
    return image_object(lena(),colormap='grey')

def _InitObjects():
    # 100 points sin function, plotted as green circles
    data1 = 2.*pi*arange(200)/200.
    data1.shape = (100, 2)
    data1[:,1] = sin(data1[:,0])
    #markers1 = poly_marker(data1, color='green', marker='circle',size=1)
    markers1 = line_object(data1)
    
    # 50 points cos function, plotted as red line
    data1 = 2.*pi*arange(100)/100.
    data1.shape = (50,2)
    data1[:,1] = cos(data1[:,0])
    #lines = poly_line(data1, color='red')
    lines = line_object(data1)
    # A few more points...
    #markers2 = poly_marker([(0., 0.), (pi/4., 1.), (pi/2, 0.),
    #                      (3.*pi/4., -1)], color='blue',
    #                      fillcolor='green', marker='cross')
    markers2 = line_object([(0., 0.), (pi/4., 1.), (pi/2, 0.),(3.*pi/4., -1)])
    # An Image
    return [markers1]#, lines, markers2]


def test_axis():
    a = axis_object(rotate = 0)
    graph_area = box_object((10,10),(100,100))
    bounds = (-1.,1.)
    a.calculate_ticks(bounds)
    dummy_dc = 0
    a.layout(graph_area,dummy_dc)
    print a.tick_points
    
    bounds = (0.,1.)
    a.calculate_ticks(bounds)
    a.layout(graph_area,dummy_dc)
    print a.tick_points
    print a.tick_start
    print a.tick_stop



if __name__ == '__main__':
        
    class MyApp(wx.wxApp):
        def OnInit(self):
            frame = plot_frame(wx.NULL, -1, "Graph",size=(400,400))
            frame.Show(wx.TRUE)
            self.SetTopWindow(frame)
            #frame.OnPlotDraw(None)
            return wx.TRUE


    app = MyApp(0)
    app.MainLoop()



#!/usr/bin/env python

import string,os,sys
if sys.version[:3]>='2.3':
    import re
else:
    import pre as re
from distutils.dir_util import mkpath

def all_subroutines(interface_in):
    # remove comments
    comment_block_exp = re.compile(r'/\*(?:\s|.)*?\*/')
    subroutine_exp = re.compile(r'subroutine (?:\s|.)*?end subroutine.*')
    function_exp = re.compile(r'function (?:\s|.)*?end function.*')
    
    interface = comment_block_exp.sub('',interface_in)
    subroutine_list = subroutine_exp.findall(interface)
    function_list = function_exp.findall(interface)
    subroutine_list = subroutine_list + function_list 
    subroutine_list = map(lambda x: string.strip(x),subroutine_list)
    return subroutine_list

def real_convert(val_string):
    return val_string

def complex_convert(val_string):
    return '(' + val_string + ',0.)'

def convert_types(interface_in,converter):
    regexp = re.compile(r'<type_convert=(.*?)>')
    interface = interface_in[:]
    while 1:
        sub = regexp.search(interface)
        if sub is None: break
        converted = converter(sub.group(1))
        interface = string.replace(interface,sub.group(),converted)
    return interface

def generic_expand(generic_interface,skip_names=[]):
    generic_types ={'s' :('real',            'real', real_convert,
                          'real'), 
                    'd' :('double precision','double precision',real_convert,
                          'double precision'),
                    'c' :('complex',         'complex',complex_convert,
                          'real'), 
                    'z' :('double complex',  'double complex',complex_convert,
                          'double precision'),                   
                    'cs':('complex',         'real',complex_convert,
                          'real'),
                    'zd':('double complex',  'double precision',complex_convert,
                          'double precision'),
                    'sc':('real',            'complex',real_convert,
                          'real'),
                    'dz':('double precision','double complex', real_convert,
                          'double precision')}
    generic_c_types = {'real':'float',
                       'double precision':'double',
                       'complex':'complex_float',
                       'double complex':'complex_double'}
    # cc_types is specific in ATLAS C BLAS, in particular, for complex arguments
    generic_cc_types = {'real':'float',
                       'double precision':'double',
                       'complex':'void',
                       'double complex':'void'}
    #2. get all subroutines    
    subs = all_subroutines(generic_interface)
    print len(subs)
    #loop through the subs
    type_exp = re.compile(r'<tchar=(.*?)>')
    TYPE_EXP = re.compile(r'<TCHAR=(.*?)>')
    routine_name = re.compile(r'(subroutine|function)\s*(?P<name>\w+)\s*\(')
    interface = ''
    for sub in subs:
        #3. Find the typecodes to use:
        m = type_exp.search(sub)
        if m is None:
            interface = interface + '\n\n' + sub
            continue
        type_chars = m.group(1)
        # get rid of spaces
        type_chars = string.replace(type_chars,' ','')
        # get a list of the characters (or character pairs)
        type_chars = string.split(type_chars,',')
        # Now get rid of the special tag that contained the types
        sub = re.sub(type_exp,'<tchar>',sub)
        m = TYPE_EXP.search(sub)
        if m is not None:
            sub = re.sub(TYPE_EXP,'<TCHAR>',sub)
        sub_generic = string.strip(sub)
        for char in type_chars:            
            type_in,type_out,converter, rtype_in = generic_types[char]
            sub = convert_types(sub_generic,converter)
            function_def = string.replace(sub,'<tchar>',char)
            function_def = string.replace(function_def,'<TCHAR>',string.upper(char))

            function_def = string.replace(function_def,'<type_in>',type_in)
            function_def = string.replace(function_def,'<type_in_c>',
                                          generic_c_types[type_in])
            function_def = string.replace(function_def,'<type_in_cc>',
                                          generic_cc_types[type_in])
            
            function_def = string.replace(function_def,'<rtype_in>',rtype_in)
            function_def = string.replace(function_def,'<rtype_in_c>',
                                          generic_c_types[rtype_in])

            function_def = string.replace(function_def,'<type_out>',type_out)
            function_def = string.replace(function_def,'<type_out_c>',
                                          generic_c_types[type_out])
            m = routine_name.match(function_def)
            if m:
                if m.group('name') in skip_names:
                    print 'Skipping',m.group('name')
                    continue
            else:
                print 'Possible bug: Failed to determine routines name'
            interface = interface + '\n\n' + function_def

    return interface

#def interface_to_module(interface_in,module_name,include_list,sdir='.'):
def interface_to_module(interface_in,module_name):
    pre_prefix = "!%f90 -*- f90 -*-\n"
    # heading and tail of the module definition.
    file_prefix = "\npython module " + module_name +" ! in\n" \
                  "!usercode '''#include \"cblas.h\"\n"\
                  "!'''\n"\
                  "    interface  \n"
    file_suffix = "\n    end interface\n" \
             "end module %s" % module_name
    return  pre_prefix + file_prefix + interface_in + file_suffix

def process_includes(interface_in,sdir='.'):
    include_exp = re.compile(r'\n\s*[^!]\s*<include_file=(.*?)>')
    include_files = include_exp.findall(interface_in)
    for filename in include_files:
        f = open(os.path.join(sdir,filename))
        interface_in = string.replace(interface_in,
                                      '<include_file=%s>'%filename,
                                      f.read())
        f.close()
    return interface_in

def generate_interface(module_name,src_file,target_file,skip_names=[]):
    print "generating",module_name,"interface"
    f = open(src_file)
    generic_interface = f.read()
    f.close()
    sdir = os.path.dirname(src_file)
    generic_interface = process_includes(generic_interface,sdir)
    generic_interface = generic_expand(generic_interface,skip_names)
    module_def = interface_to_module(generic_interface,module_name)
    mkpath(os.path.dirname(target_file))
    f = open(target_file,'w')
    user_routines = os.path.join(sdir,module_name+"_user_routines.pyf")
    if os.path.exists(user_routines):
        f2 = open(user_routines)
        f.write(f2.read())
        f2.close()
    f.write(module_def)
    f.close()

def process_all():
    # process the standard files.
    for name in ['fblas','cblas','clapack','flapack']:
        generate_interface(name,'generic_%s.pyf'%(name),name+'.pyf')

    
if __name__ == "__main__":
    process_all()



import sys
sys.path.insert(0,'..')
import inline_tools

import time

def print_compare(n):
    print 'Printing %d integers:'%n
    t1 = time.time()
    for i in range(n):
        print i,
    t2 = time.time()
    py = (t2-t1)
    
    # get it in cache
    inline_tools.inline('printf("%d",i);',['i'])
    t1 = time.time()
    for i in range(n):
        inline_tools.inline('printf("%d",i);',['i'])
    t2 = time.time()
    print ' speed in python:', py
    print ' speed in c:',(t2 - t1)    
    print ' speed up: %3.2f' % (py/(t2-t1))

def cout_example(lst):
    # get it in cache
    i = lst[0]
    inline_tools.inline('std::cout << i << std::endl;',['i'])
    t1 = time.time()
    for i in lst:
        inline_tools.inline('std::cout << i << std::endl;',['i'])
    t2 = time.time()
    
if __name__ == "__main__":
    n = 3000
    print_compare(n)    
    print "calling cout with integers:"
    cout_example([1,2,3])
    print "calling cout with strings:"
    cout_example(['a','bb', 'ccc'])


""" Test functions for basic module

"""

import unittest
import scipy_base.limits as limits
from scipy_test.testing import assert_array_equal, assert_equal
from scipy_test.testing import assert_almost_equal, assert_array_almost_equal
from scipy import *

##################################################

### Test for rand

class test_rand(unittest.TestCase):
    def __init__(self,*args,**kwds):
        unittest.TestCase.__init__(self,*args,**kwds)
        self.z = rand(100,10,20)
        
    def check_shape(self):
        assert_equal(self.z.shape,(100,10,20))

    def check_mean(self):
        mn = add.reduce(ravel(self.z)) / 200 / 100.0
        assert_almost_equal(mn,0.5,2)

    def check_std(self):
        std = add.reduce((ravel(self.z)-0.5)**2) / 200.0 / 100.0
        assert_almost_equal(std,1.0/12.0,2)

class test_randn(unittest.TestCase):
    def __init__(self,*args,**kwds):
        unittest.TestCase.__init__(self,*args,**kwds)
        self.z = randn(100,10,20)
        
    def check_shape(self):
        assert_equal(self.z.shape,(100,10,20))

    def check_mean(self):
        mn = add.reduce(ravel(self.z)) / 200 / 100.0
        assert_almost_equal(mn,0.0,1)

    def check_std(self):
        std = sqrt(add.reduce(ravel(self.z)**2) / 200.0 / 100.0)
        assert_almost_equal(std,1.0,1)


val = limits.double_resolution

class test_eye(unittest.TestCase):
    def check_basic(self):
        assert_equal(eye(4),array([[1,0,0,0],
                                   [0,1,0,0],
                                   [0,0,1,0],
                                   [0,0,0,1]]))
        assert_equal(eye(4,typecode='f'),array([[1,0,0,0],
                                                [0,1,0,0],
                                                [0,0,1,0],
                                                [0,0,0,1]],'f'))
    def check_diag(self):
        assert_equal(eye(4,k=1),array([[0,1,0,0],
                                       [0,0,1,0],
                                       [0,0,0,1],
                                       [0,0,0,0]]))
        assert_equal(eye(4,k=-1),array([[0,0,0,0],
                                        [1,0,0,0],
                                        [0,1,0,0],
                                        [0,0,1,0]]))
    def check_2d(self):
        assert_equal(eye(4,3),array([[1,0,0],
                                     [0,1,0],
                                     [0,0,1],
                                     [0,0,0]]))
        assert_equal(eye(3,4),array([[1,0,0,0],
                                     [0,1,0,0],
                                     [0,0,1,0]]))        
    def check_diag2d(self):
        assert_equal(eye(3,4,k=2),array([[0,0,1,0],
                                         [0,0,0,1],
                                         [0,0,0,0]]))
        assert_equal(eye(4,3,k=-2),array([[0,0,0],
                                          [0,0,0],
                                          [1,0,0],
                                          [0,1,0]]))

class test_tri(unittest.TestCase):
    def check_basic(self):
        assert_equal(tri(4),array([[1,0,0,0],
                                   [1,1,0,0],
                                   [1,1,1,0],
                                   [1,1,1,1]]))
        assert_equal(tri(4,typecode='f'),array([[1,0,0,0],
                                                [1,1,0,0],
                                                [1,1,1,0],
                                                [1,1,1,1]],'f'))
    def check_diag(self):
        assert_equal(tri(4,k=1),array([[1,1,0,0],
                                       [1,1,1,0],
                                       [0,1,1,1],
                                       [1,1,1,1]]))
        assert_equal(tri(4,k=-1),array([[0,0,0,0],
                                        [1,0,0,0],
                                        [1,1,0,0],
                                        [1,1,1,0]]))
    def check_2d(self):
        assert_equal(tri(4,3),array([[1,0,0],
                                     [1,1,0],
                                     [1,1,1],
                                     [1,1,1]]))
        assert_equal(tri(3,4),array([[1,0,0,0],
                                     [1,1,0,0],
                                     [1,1,1,0]]))        
    def check_diag2d(self):
        assert_equal(tri(3,4,k=2),array([[1,1,1,0],
                                         [1,1,1,1],
                                         [1,1,1,1]]))
        assert_equal(tri(4,3,k=-2),array([[0,0,0],
                                          [0,0,0],
                                          [1,0,0],
                                          [1,1,0]]))

class test_diag(unittest.TestCase):
    def check_vector(self):
        vals = (100*rand(5)).astype('l')
        b = zeros((5,5))
        for k in range(5):
            b[k,k] = vals[k]
        assert_equal(diag(vals),b)
        b = zeros((7,7))
        c = b.copy()
        for k in range(5):
            b[k,k+2] = vals[k]
            c[k+2,k] = vals[k]
        assert_equal(diag(vals,k=2), b)
        assert_equal(diag(vals,k=-2), c)

    def check_matrix(self):
        vals = (100*rand(5,5)+1).astype('l')
        b = zeros((5,))
        for k in range(5):
            b[k] = vals[k,k]
        assert_equal(diag(vals),b)
        b = b*0
        for k in range(3):
            b[k] = vals[k,k+2]
        assert_equal(diag(vals,2),b[:3])
        for k in range(3):
            b[k] = vals[k+2,k]
        assert_equal(diag(vals,-2),b[:3])

class test_fliplr(unittest.TestCase):
    def check_basic(self):
        self.failUnlessRaises(ValueError, fliplr, ones(4))        
        self.failUnlessRaises(ValueError, fliplr, ones((4,3,2)))
        a = rand(4,4)
        b = a[:,::-1]
        assert_equal(fliplr(a),b)
        a = [[0,1,2],
             [3,4,5]]
        b = [[2,1,0],
             [5,4,3]]
        assert_equal(fliplr(a),b)

class test_flipud(unittest.TestCase):
    def check_basic(self):
        self.failUnlessRaises(ValueError, flipud, ones(4))
        self.failUnlessRaises(ValueError, flipud, ones((4,3,2)))
        a = rand(4,4)
        b = a[::-1,:]
        assert_equal(flipud(a),b)
        a = [[0,1,2],
             [3,4,5]]
        b = [[3,4,5],
             [0,1,2]]
        assert_equal(flipud(a),b)

class test_rot90(unittest.TestCase):
    def check_basic(self):
        self.failUnlessRaises(ValueError, rot90, ones(4))
        self.failUnlessRaises(ValueError, rot90, ones((4,3,2)))

        a = [[0,1,2],
             [3,4,5]]
        b1 = [[2,5],
              [1,4],
              [0,3]]
        b2 = [[5,4,3],
              [2,1,0]]
        b3 = [[3,0],
              [4,1],
              [5,2]]
        b4 = [[0,1,2],
              [3,4,5]]

        for k in range(-3,13,4):
            assert_equal(rot90(a,k=k),b1)
        for k in range(-2,13,4):
            assert_equal(rot90(a,k=k),b2)
        for k in range(-1,13,4):
            assert_equal(rot90(a,k=k),b3)
        for k in range(0,13,4):
            assert_equal(rot90(a,k=k),b4)

class test_tril(unittest.TestCase):
    def check_basic(self):
        a = (100*rand(5,5)).astype('l')
        b = a.copy()
        for k in range(5):
            for l in range(k+1,5):
                b[k,l] = 0
        assert_equal(tril(a),b)

    def check_diag(self):        
        a = (100*rand(5,5)).astype('f')
        b = a.copy()
        for k in range(5):
            for l in range(k+3,5):
                b[k,l] = 0
        assert_equal(tril(a,k=2),b)
        b = a.copy()
        for k in range(5):
            for l in range(max((k-1,0)),5):
                b[k,l] = 0
        assert_equal(tril(a,k=-2),b)

class test_triu(unittest.TestCase):
    def check_basic(self):
        a = (100*rand(5,5)).astype('l')
        b = a.copy()
        for k in range(5):
            for l in range(k+1,5):
                b[l,k] = 0
        assert_equal(triu(a),b)

    def check_diag(self):        
        a = (100*rand(5,5)).astype('f')
        b = a.copy()
        for k in range(5):
            for l in range(max((k-1,0)),5):
                b[l,k] = 0
        assert_equal(triu(a,k=2),b)
        b = a.copy()
        for k in range(5):
            for l in range(k+3,5):
                b[l,k] = 0
        assert_equal(tril(a,k=-2),b)

class test_amax(unittest.TestCase):
    def check_basic(self):
        a = [3,4,5,10,-3,-5,6.0]
        assert_equal(amax(a),10.0)
        b = [[3,6.0, 9.0],
             [4,10.0,5.0],
             [8,3.0,2.0]]
        assert_equal(amax(b),[8.0,10.0,9.0])
        assert_equal(amax(b,axis=1),[9.0,10.0,8.0])
        
class test_amin(unittest.TestCase):
    def check_basic(self):
        a = [3,4,5,10,-3,-5,6.0]
        assert_equal(amin(a),-5.0)
        b = [[3,6.0, 9.0],
             [4,10.0,5.0],
             [8,3.0,2.0]]
        assert_equal(amin(b),[3.0,3.0,2.0])
        assert_equal(amin(b,axis=1),[3.0,4.0,2.0])


class test_ptp(unittest.TestCase):
    def check_basic(self):
        a = [3,4,5,10,-3,-5,6.0]
        assert_equal(ptp(a),15.0)
        b = [[3,6.0, 9.0],
             [4,10.0,5.0],
             [8,3.0,2.0]]
        assert_equal(ptp(b),[5.0,7.0,7.0])
        assert_equal(ptp(b,axis=1),[6.0,6.0,6.0])


class test_cumsum(unittest.TestCase):
    def check_basic(self):
        ba = [1,2,10,11,6,5,4]
        ba2 = [[1,2,3,4],[5,6,7,9],[10,3,4,5]]
        for ctype in ['1','b','s','i','l','f','d','F','D']:
            a = array(ba,ctype)
            a2 = array(ba2,ctype)
            assert_array_equal(cumsum(a), array([1,3,13,24,30,35,39],ctype))
            assert_array_equal(cumsum(a2), array([[1,2,3,4],[6,8,10,13],
                                                  [16,11,14,18]],ctype))
            assert_array_equal(cumsum(a2,axis=1),
                               array([[1,3,6,10],
                                      [5,11,18,27],
                                      [10,13,17,22]],ctype))

class test_prod(unittest.TestCase):
    def check_basic(self):
        ba = [1,2,10,11,6,5,4]
        ba2 = [[1,2,3,4],[5,6,7,9],[10,3,4,5]]
        for ctype in ['1','b','s','i','l','f','d','F','D']:
            a = array(ba,ctype)
            a2 = array(ba2,ctype)
            if ctype in ['1', 'b']:
                self.failUnlessRaises(ArithmeticError, prod, a)
                self.failUnlessRaises(ArithmeticError, prod, a2, 1)
                self.failUnlessRaises(ArithmeticError, prod, a)
            else:                
                assert_equal(prod(a),26400)
                assert_array_equal(prod(a2), array([50,36,84,180],ctype))
                assert_array_equal(prod(a2,axis=1),
                                   array([24, 1890, 600],ctype))

class test_cumprod(unittest.TestCase):
    def check_basic(self):
        ba = [1,2,10,11,6,5,4]
        ba2 = [[1,2,3,4],[5,6,7,9],[10,3,4,5]]
        for ctype in ['1','b','s','i','l','f','d','F','D']:
            a = array(ba,ctype)
            a2 = array(ba2,ctype)
            if ctype in ['1', 'b']:
                self.failUnlessRaises(ArithmeticError, cumprod, a)
                self.failUnlessRaises(ArithmeticError, cumprod, a2, 1)
                self.failUnlessRaises(ArithmeticError, cumprod, a)
            else:                
                assert_array_equal(cumprod(a),
                                   array([1, 2, 20, 220,
                                          1320, 6600, 26400],ctype))
                assert_array_equal(cumprod(a2),
                                   array([[ 1,  2,  3,   4],
                                          [ 5, 12, 21,  36],
                                          [50, 36, 84, 180]],ctype))
                assert_array_equal(cumprod(a2,axis=1),
                                   array([[ 1,  2,   6,   24],
                                          [ 5, 30, 210, 1890],
                                          [10, 30, 120,  600]],ctype))


class test_diff(unittest.TestCase):
    def check_basic(self):
        x = [1,4,6,7,12]
        out = array([3,2,1,5])
        out2 = array([-1,-1,4])
        out3 = array([0,5])
        assert_array_equal(diff(x),out)
        assert_array_equal(diff(x,n=2),out2)
        assert_array_equal(diff(x,n=3),out3)

    def check_nd(self):
        x = 20*rand(10,20,30)
        out1 = x[:,:,1:] - x[:,:,:-1]
        out2 = out1[:,:,1:] - out1[:,:,:-1]
        out3 = x[1:,:,:] - x[:-1,:,:]
        out4 = out3[1:,:,:] - out3[:-1,:,:]
        assert_array_equal(diff(x),out1)
        assert_array_equal(diff(x,n=2),out2)
        assert_array_equal(diff(x,axis=0),out3)
        assert_array_equal(diff(x,n=2,axis=0),out4)


class test_cov(unittest.TestCase):
    def check_basic(self):
        x = [[1,1.3],
             [1.1,1.5],
             [0.9,1.8]]
        c = cov(x)
        tmp = [[0.0100, -0.0150],[-0.0150, 0.0633]]        
        assert_array_almost_equal(c,tmp,4)

    def check_twoargs(self):
        x = [1,1.1,0.9]
        y = [1.3,1.5,1.8]
        c = cov(x,y)
        z = [[1,1.3],
             [1.1,1.5],
             [0.9,1.8]]
        d = cov(z)
        e = cov([x,y],rowvar=1)
        tmp = [[0.0100, -0.0150],[-0.0150, 0.0633]]
        assert_array_almost_equal(c,tmp,4)
        assert_array_almost_equal(c,d,11)
        assert_array_almost_equal(e,d,11)

    def check_bias(self):
        z = [[1,1.3],
             [1.1,1.5],
             [0.9,1.8]]
        d = cov(z,bias=1)
        tmp = [[0.0067,-0.0100],[-0.0100, 0.0422]]
        assert_array_almost_equal(d,tmp,4)

class test_corrcoef(unittest.TestCase):
    def check_basic(self):
        x = [[1,1.3],
             [1.1,1.5],
             [0.9,1.8]]
        c = corrcoef(x)
        tmp = [[1.0000, -0.5960],[-0.5960, 1.0000]]
        assert_array_almost_equal(c,tmp,4)

    def check_twoargs(self):
        x = [1,1.1,0.9]
        y = [1.3,1.5,1.8]
        c = corrcoef(x,y)
        z = [[1,1.3],
             [1.1,1.5],
             [0.9,1.8]]
        d = corrcoef(z)
        e = corrcoef([x,y],rowvar=1)
        tmp = [[1.0000, -0.5960],[-0.5960, 1.0000]]
        assert_array_almost_equal(c,tmp,4)
        assert_array_almost_equal(c,d,11)
        assert_array_almost_equal(e,d,11)

class test_squeeze(unittest.TestCase):
    def check_basic(self):
        a = rand(20,10,10,1,1)
        b = rand(20,1,10,1,20)
        c = rand(1,1,20,10)
        assert_array_equal(squeeze(a),reshape(a,(20,10,10)))
        assert_array_equal(squeeze(b),reshape(b,(20,10,20)))
        assert_array_equal(squeeze(c),reshape(c,(20,10)))

class test_sinc(unittest.TestCase):
    def check_basic(self):
        x = r_[grid[-10:0:50j],grid[0:10:50j]]
        y = sinc(x)
        tmp = x*pi
        tmp = sin(tmp)/tmp
        tmp[50] = 1.0
        tmp[49] = 1.0
        assert_array_almost_equal(y,tmp,11)
        
class test_angle(unittest.TestCase):
    def check_basic(self):
        x = [1+3j,sqrt(2)/2.0+1j*sqrt(2)/2,1,1j,-1,-1j,1-3j,-1+3j]
        y = angle(x)
        yo = [arctan(3.0/1.0),arctan(1.0),0,pi/2,pi,-pi/2.0,-arctan(3.0/1.0),pi-arctan(3.0/1.0)]
        z = angle(x,deg=1)
        zo = array(yo)*180/pi
        assert_array_almost_equal(y,yo,11)
        assert_array_almost_equal(z,zo,11)
        
##################################################

def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append( unittest.makeSuite(test_rand,'check_') )
        suites.append( unittest.makeSuite(test_randn,'check_') )
        suites.append( unittest.makeSuite(test_eye,'check_') )
        suites.append( unittest.makeSuite(test_tri,'check_') )
        suites.append( unittest.makeSuite(test_diag,'check_') )
        suites.append( unittest.makeSuite(test_fliplr,'check_') )
        suites.append( unittest.makeSuite(test_flipud,'check_') )
        suites.append( unittest.makeSuite(test_rot90,'check_') )
        suites.append( unittest.makeSuite(test_triu,'check_') )
        suites.append( unittest.makeSuite(test_tril,'check_') )
        suites.append( unittest.makeSuite(test_amax,'check_') )
        suites.append( unittest.makeSuite(test_amin,'check_') )
        suites.append( unittest.makeSuite(test_ptp,'check_') )
        suites.append( unittest.makeSuite(test_mean,'check_') )
        suites.append( unittest.makeSuite(test_median,'check_') )
        suites.append( unittest.makeSuite(test_std,'check_') )
        suites.append( unittest.makeSuite(test_cumsum,'check_') )
        suites.append( unittest.makeSuite(test_prod,'check_') )
        suites.append( unittest.makeSuite(test_cumprod,'check_') )
        suites.append( unittest.makeSuite(test_trapz,'check_') )
        suites.append( unittest.makeSuite(test_diff,'check_') )
        suites.append( unittest.makeSuite(test_cov,'check_') )
        suites.append( unittest.makeSuite(test_corrcoef,'check_') )
        suites.append( unittest.makeSuite(test_squeeze,'check_') )
        suites.append( unittest.makeSuite(test_sinc,'check_') )
        suites.append( unittest.makeSuite(test_angle,'check_') )
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite(level)
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner


if __name__ == "__main__":
    test()



#genetic algorithm population
#based on galib. 
import time
import ga_list 
import re, copy
import scaling
import selection
import Numeric
import scipy.stats as stats
from ga_util import *
import pdb

def ftn_minimize(x,y): 
	"""Minimization comparator for fitness (scaled score)."""
	return cmp(x.fitness(),y.fitness())
def ftn_maximize(x,y): 
	"""Maximization comparator for fitness (scaled score)."""
	return cmp(y.fitness(),x.fitness())
def sc_minimize(x,y): 
	"""Minimization comparator for raw score."""
#	return cmp(x.score(),y.score())
	#removed one function call
	return cmp(x.evaluate(),y.evaluate())
def sc_maximize(x,y): 
	"""Maximization comparator for raw score."""
#	return cmp(y.score(),x.score())
	return cmp(y.evaluate(),x.evaluate())
class default_pop_evaluator:
	"""The **evaluate()** method simply calls the **evaluate()**
	   method for all the genomes in the population
	"""
	def evaluate(self,pop,force = 0):
		try:
			evals = 0
			if not pop.evaluated or force:
				for ind in pop: ind.evaluate(force)
		except:
			#this makes where a pop evaluator can simply evaluate a list
			#of genomes - might be useful to simplify remote evaluation
			for ind in pop: ind.evaluate(force)
			
class default_pop_initializer:
	"""The **evaluate()** method simply calls the **evaluate()**
	   method for all the genomes in the population
	"""
	def evaluate(self,pop,settings):
		for i in pop: i.initialize(settings)			
		
class population(ga_list.ga_list):
	"""A population of genomes.  The population is constructed by cloning a 
	   single genome multiple times.
	   
	   The population can be treated, for the most part, like a python list 
	   because it is derived from ga_list.ga_list.  This allows the addition 
	   and subtraction of individuals from the population using list 
	   operators such as slice, append, etc.
	   
         Some of the methods, such as **best()** and **worst()**, assume that 
         the population is sorted.
          
	   **Attributes:**
	   
	   stats -- dictionary of dictionaries. The statistics for the population.
	            This keeps up with such things as the number of evaluations,
	            the best current score, the best ever score, etc.
	   scaler -- An instance of a scaler class.  Used to scale the raw scores of
	             the population to generate the genome fitness values.  If you don't
	             want scaling then use the class **scaling.no_scaling**.  The
	             default scaler is **sigma_truncation_scaling** because it handles
	             negative fitness scores effectively.
	   evaluator -- An instance of an evaluator class.  Used to evaluate the individuals
	                of the population.
	   selector -- An instance of a selector class.  Used to select the individuals
	               of the population that go into the breeding pool.  The default
	               selector is **srs_selector**.
	               
	   **Flags (0 or 1):**
	   
	   evaluated -- Population has been avaluated
	   scaled -- Population has been scaled
	   sorted -- Population is sorted from best to worst
	   select_ready -- The Selector has been updated and is ready for selections
 	   stated -- The population statistics are up to date.            
	"""
	default_scaler = scaling.sigma_truncation_scaling
	default_selector = selection.srs_selector
	default_evaluator =  default_pop_evaluator
	default_initializer =  default_pop_initializer
	
	scaler = default_scaler()
	evaluator = default_evaluator()
	initializer = default_initializer()
#	selector = default_selector()

	def __init__(self,genome,size=1):
		"""Arguments:
		
		   genome -- a genome object.
		   size -- number.  The population size.  The genome will be 
		           replicated size times to fill the population.
		"""
		self.model_genome = genome
		ga_list.ga_list.__init__(self)
		self.ftn_comparator = ftn_maximize
		self.sc_comparator = sc_maximize		
		self._size(size)
		self.selector = population.default_selector() #why'd I do this?
		self.stats={}
	def initialize(self,settings = None):
		"""This method **must** be called before a genetic algorithm 
		   begins evolving the population.  It takes care of initializing
		   the individual genomes, evaluating them, and scaling the population.
		   It also clears and intializes the statistics for the population.
		   
		   Arguments:
		
		   settings -- dictionary of genetic algorithm parameters.  These
		               are passed on to the genomes for initialization.
		"""	
		self.stats = {'current':{},'initial':{},'overall':{}}
		self.stats['ind_evals'] = 0

		print "beigninning genome generation" 
		b = time.clock()	
		self.initializer.evaluate(self,settings)
		e = time.clock()	
		print "finished generation: ", e-b	
		self.touch(); 
		b = time.clock()	
		self.evaluate()
		e = time.clock()	
		print "evaluation time: ", e-b	
		self.scale()
		self.update_stats()
		self.stats['initial']['avg'] = self.stats['current']['avg']
		self.stats['initial']['max'] = self.stats['current']['max']
		self.stats['initial']['min'] = self.stats['current']['min']
		self.stats['initial']['dev'] = self.stats['current']['dev']
	def clone(self): 
		"""Returns a population that has a shallow copy the all the 
		   attributes and clone of all the genomes in the original 
		   object.  It also makes a deep copy of the stats dictionary.
		"""	
		new = ga_list.ga_list.data_clone(self)
		new.stats = {}
		new.stats.update(self.stats)
		return new
	def touch(self):
		"""Reset all the flags for the population."""
		self.evaluated = 0; self.scaled = 0; self.sorted = 0; self.select_ready = 0
		self.stated = 0
	def _size(self, l):
		"""Resize the population."""
		del self[l:len(self)]
		for i in range(len(self),l): self.append(self.model_genome.clone())
		return len(self)		
	def evaluate(self, force = 0):
		"""Call the **evaluator.evaluate()** method to evaluate
		   the population.  The population is also sorted so that 
		   it maintains the correct order.  The population is only 
		   updated if *evaluated=0*.
		   
		   Arguments:
		   
		   force -- forces evaluation even if evaluated = 1
		"""
		b = time.clock()
		self.evaluator.evaluate(self,force)
		e1 = time.clock()
		self.sort()
		e2 = time.clock()
		#this is a cluge to get eval count to work correctly
		preval = self.stats['ind_evals']
		for ind in self:  
			self.stats['ind_evals'] = self.stats['ind_evals'] + ind.evals
			ind.evals = 0		
		print 'evals: ', self.stats['ind_evals'] - preval
		e3 = time.clock()
		self.touch()
		self.evaluated = 1
		e4 = time.clock()
		#print 'eval:',e1-b, 'sort:',e2-e1, 'stats:',e3-e2, 'touch:',e4-e3
	def mutate(self):
		mutations = 0
		for ind in self: mutations  =  mutations + ind.mutate()
		return mutations
	
	def sort(self,type = 'raw', force = 0):
		"""Sort the population so they are ordered from best
		   to worst.  This ordering is specified by the comparator
		   operator used to sort the population.  The comparator
		   is specified usign the **min_or_max()** function. 
		   
		   Arguments:
		   
		   type -- 'raw' or 'scaled'.  Determines wether the
		           sorting is done based on raw scores or on
		           fitness (scaled) scores.
		   force -- forces the sort even if sorted = 1
		"""
#		if not self.sorted or force: 
		if(type == 'scaled'): self.data.sort(self.ftn_comparator)
		elif(type == 'raw'): self.data.sort(self.sc_comparator)	
		else: raise GAError, 'sort type must be "scaled" or "raw"'			
		self.sorted = 1
	def select(self, cnt = 1):
		"""Calls the selector and returns *cnt* individuals.
		
		   Arguments:
		   
		   cnt -- The number of individuals to return.
		"""
		if not self.select_ready:
			self.selector.update(self)
			self.select_ready = 1
		return self.selector.select(self,cnt)
	def scale(self, force = 0):
		"""Calls the **scaler.scale()** method and updates
		   the fitness of each individual.

 		   Arguments:
		   
		   force -- forces the scaling even if scaled = 1
		"""		   
		if not (self.scaled or force):
			self.scaler.scale(self)			
		self.scaled = 1
	def fitnesses(self): 
		"""Returns the fitness (scaled score) of all the
		   individuals in a population as a Numeric array.
		"""		   
		return Numeric.array(map(lambda x: x.fitness(),self))	
	def scores(self):	
		"""Returns the scores (raw) of all the
		   individuals in a population as a Numeric array.
		"""		   
		return Numeric.array(map(lambda x: x.score(),self))	
	def best(self, ith_best = 1): 
		"""Returns the best individual in the population.
		   *It assumes the population has been sorted.*

 		   Arguments:
		   
		   ith_best -- Useful if you want the second(2), third(3), etc.
		               best individual in the population.
		"""		   
		return self[ith_best - 1]
	def worst(self,ith_worst = 1): 
		"""Returns the worst individual in the population.
		   *It assumes the population has been sorted.*

 		   Arguments:
		   
		   ith_worst -- Useful if you want the second(2), third(3), etc.
		               worst individual in the population.
		"""		   
		return self[-ith_worst]		
	def min_or_max(self,*which_one):
		"""Returns or set 'min' or 'max' indicating whether the
		   population is to be minimized or maximized.  
		   *Minimization may require some special handling 
		   in the scaling and selector routines.*

 		   Arguments:
		   
		   which_one -- 'min' or 'max'(optional). Tells the population
		                the problem is a minimization or maximizization
		                problem.
		"""		   
		if len(which_one): 
			if (re.match('min.*',which_one[0],re.I)):
				self.ftn_comparator = ftn_minimize
				self.sc_comparator = sc_minimize				
			elif (re.match('max.*',which_one[0],re.I)):
				self.ftn_comparator = ftn_maximize
				self.sc_comparator = sc_maximize								
			else:	raise GaError, "min_or_max expects 'min' or 'max'"
		if self.ftn_comparator == ftn_minimize: return 'min'
		elif self.ftn_comparator == ftn_maximize: return 'max'
	def update_stats(self):
		"""Update the statistics for the population."""
		s = self.scores()
		self.stats['current']['max'] = max(s)
		self.stats['current']['avg'] = my_mean(s)
		self.stats['current']['min'] = min(s)
		if len(s) > 1: self.stats['current']['dev'] = my_std(s)
		else: self.stats['current']['dev'] = 0	
		try: self.stats['overall']['max'] = max(self.stats['overall']['max'],
								    self.stats['current']['max'])
		except KeyError: self.stats['overall']['max'] = self.stats['current']['max']
		try: self.stats['overall']['min'] = min(self.stats['overall']['min'],
								    self.stats['current']['min'])
		except KeyError: self.stats['overall']['min'] = self.stats['current']['min']
